@@ -1,6 +1,5 @@
 ;;; This file defines the cig foreign function interface for Scheme 48.
 ;;; The current version is Cig 3.0.
-
 ;;; This file contains the following Scheme 48 modules:
 ;;; - cig-processor
 ;;;   The code for translating DEFINE-FOREIGN forms into C stubs.
@@ -29,7 +28,7 @@
   (access signals) ; for ERROR
   (begin
     (define error (structure-ref signals error))
-
+    
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; The general syntax of define-foreign is:
 ;;; (define-foreign scheme-name (c-name arg1 ... argn) [no-declare]
@@ -142,45 +141,57 @@
 ;;; closed in the macro expander's environment, so the user won't lose
 ;;; if he should accidentally bind INTEGER? to something unusual, and
 ;;; then try a DEFINE-FOREIGN.
+
+;;; WARNING: the xxx_t are not yet adapted to the new FFI, only time_t, 
+;;; long and int are entered via enter/extract_integer, which converts to a 
+;;; bignum if appropriate.
+;;; Since this requires runtime-checks it is slower than the _integer's
+;;; You have to check, where more than 29 bits is required
+
 (define *simple-argrep-alist* '(
 
- (char   #("char ~a"	char?	"EXTRACT_CHAR"  #f))
+ (char   #("char ~a"	char?	"s48_extract_char"  #f))
  (bool   #("int ~a"	#f	"EXTRACT_BOOLEAN"  #f))
 
- (integer  #("int ~a" 		  integer?   "EXTRACT_FIXNUM"   #f))
- (short_u  #("unsigned short ~a"  integer?   "EXTRACT_FIXNUM"	#f))
- (size_t   #("size_t ~a"          integer?   "EXTRACT_FIXNUM"	#f))
- (mode_t   #("mode_t ~a" 	  integer?   "EXTRACT_FIXNUM"	#f))
- (gid_t    #("gid_t  ~a" 	  integer?   "EXTRACT_FIXNUM"	#f))
- (uid_t    #("uid_t ~a" 	  integer?   "EXTRACT_FIXNUM"	#f))
- (off_t    #("off_t ~a" 	  integer?   "EXTRACT_FIXNUM"	#f))
- (pid_t    #("pid_t ~a" 	  integer?   "EXTRACT_FIXNUM"	#f))
- (uint_t   #("unsigned int ~a"	  integer?   "EXTRACT_FIXNUM"	#f))
- (long     #("long ~a"		  integer?   "EXTRACT_FIXNUM"	#f))
- (fixnum   #("int ~a"	  	  fixnum?    "EXTRACT_FIXNUM"	#f))
+ (integer  #("int ~a" 		  integer?   "s48_extract_integer"   #f))
+ (short_u  #("unsigned short ~a"  integer?   "s48_extract_fixnum"	#f))
+ (size_t   #("size_t ~a"          integer?   "s48_extract_fixnum"	#f))
+ (mode_t   #("mode_t ~a" 	  integer?   "s48_extract_fixnum"	#f))
+ (gid_t    #("gid_t  ~a" 	  integer?   "s48_extract_fixnum"	#f))
+ (uid_t    #("uid_t ~a" 	  integer?   "s48_extract_fixnum"	#f))
+ (off_t    #("off_t ~a" 	  integer?   "s48_extract_fixnum"	#f))
+ (pid_t    #("pid_t ~a" 	  integer?   "s48_extract_fixnum"	#f))
+ (uint_t   #("unsigned int ~a"	  integer?   "s48_extract_integer"	#f))
+ (long     #("long ~a"		  integer?   "s48_extract_integer"	#f))
+ (fixnum   #("int ~a"	  	  fixnum?    "s48_extract_fixnum"	#f))
 
- (desc		 #("scheme_value ~a"	#f	 "" #f))
- (string-desc	 #("scheme_value ~a"	string?	 "" #f))
- (char-desc	 #("scheme_value ~a"	char?	 "" #f))
- (integer-desc	 #("scheme_value ~a"	integer? "" #f))
- (vector-desc	 #("scheme_value ~a"	vector?	 "" #f))
- (pair-desc	 #("scheme_value ~a"	pair?	 "" #f))
+ ;; extensions by JMG for the new FFI
+ (time_t  #("time_t ~a" 	  integer?   "s48_extract_integer"	#f))
+ (clock_t #("clock_t ~a"	  integer?   "s48_extract_fixnum"	#f))
+ (ssize_t #("ssize_t ~a"	  integer?   "s48_extract_fixnum"	#f))
 
- (string 	#("const char *~a" 	string? "cig_string_body" #f))
+ (desc		 #("s48_value ~a"	#f	 "" #f))
+ (string-desc	 #("s48_value ~a"	string?	 "" #f))
+ (char-desc	 #("s48_value ~a"	char?	 "" #f))
+ (integer-desc	 #("s48_value ~a"	integer? "" #f))
+ (vector-desc	 #("s48_value ~a"	vector?	 "" #f))
+ (pair-desc	 #("s48_value ~a"	pair?	 "" #f))
 
- (var-string 	#("char *~a" 		string? "cig_string_body" #f))
+ (string 	#("const char *~a" 	string? "s48_extract_string" #f))
+
+ (var-string 	#("char *~a" 		string? "s48_extract_string" #f))
 
  (string-copy 	#("char *~a" 		string? "scheme2c_strcpy" #f))))
 
 ;;; Emit C code to copy a C string into its carrier.
 (define (str-and-len->carrier carrier str)
   (format #f
-	  "{AlienVal(CAR(~a)) = (long) ~a; CDR(~a) = strlen_or_false(~a);}"
+	  "SetAlienVal(S48_CAR(~a),(long) ~a); S48_SET_CDR(~a,strlen_or_false(~a));//str-and-len"
 	  carrier str carrier str))
 
 ;;; Carrier and assignment-generator for alien values:
 (define (simple-assign carrier val)
-  (format #f "AlienVal(~a) = (long) ~a;" carrier val))
+  (format #f "SetAlienVal(~a,(long) ~a);//simple-assign" carrier val)) ;;JMG: untested
 
 ;;; Note: When MAKE-CARRIER and S-CVTR fields are taken from this table,
 ;;; they are symbols that are syntactically closed in the macro expander's
@@ -191,27 +202,34 @@
 (define *simple-retrep-alist* `(
 
  ;; All the immediate ones (we are sleazing on ints for now).
- (char   #("char ~a" "ENTER_CHAR" #f #f #f))
- (bool   #("int ~a"  "ENTER_BOOLEAN" #f #f #f))
+ (char   #("char ~a" "s48_enter_char" #f #f #f))
+ (bool   #("int ~a"  "ENTER_BOOLEAN" #f #f #f)) ;; JMG c and bool: 
+ ; s48 knows nothing about this I think
 
- (integer  #("int ~a"	         "ENTER_FIXNUM" #f #f #f))
- (fixnum   #("int ~a"	         "ENTER_FIXNUM" #f #f #f))
- (short_u  #("unsigned short ~a" "ENTER_FIXNUM" #f #f #f))
- (size_t   #("size_t ~a"	 "ENTER_FIXNUM" #f #f #f))
- (mode_t   #("mode_t ~a"	 "ENTER_FIXNUM" #f #f #f))
- (gid_t    #("gid_t  ~a"	 "ENTER_FIXNUM" #f #f #f))
- (uid_t    #("uid_t ~a"		 "ENTER_FIXNUM" #f #f #f))
- (off_t    #("off_t ~a"		 "ENTER_FIXNUM" #f #f #f))
- (pid_t    #("pid_t ~a"		 "ENTER_FIXNUM" #f #f #f))
- (uint_t   #("unsigned int ~a"	 "ENTER_FIXNUM" #f #f #f))
- (long     #("long ~a"		 "ENTER_FIXNUM" #f #f #f))
+ (integer  #("int ~a"	         "s48_enter_integer" #f #f #f))
+ (fixnum   #("int ~a"	         "s48_enter_fixnum" #f #f #f))
+ (short_u  #("unsigned short ~a" "s48_enter_fixnum" #f #f #f))
+ (size_t   #("size_t ~a"	 "s48_enter_fixnum" #f #f #f))
+ (mode_t   #("mode_t ~a"	 "s48_enter_fixnum" #f #f #f))
+ (gid_t    #("gid_t  ~a"	 "s48_enter_fixnum" #f #f #f))
+ (uid_t    #("uid_t ~a"		 "s48_enter_fixnum" #f #f #f))
+ (off_t    #("off_t ~a"		 "s48_enter_fixnum" #f #f #f))
+ (pid_t    #("pid_t ~a"		 "s48_enter_fixnum" #f #f #f))
+ (uint_t   #("unsigned int ~a"	 "s48_enter_fixnum" #f #f #f))
+ (long     #("long ~a"		 "s48_enter_integer" #f #f #f))
 
- (desc		 #("scheme_value ~a" "" #f #f #f))
- (string-desc	 #("scheme_value ~a" "" #f #f #f))
- (char-desc	 #("scheme_value ~a" "" #f #f #f))
- (integer-desc	 #("scheme_value ~a" "" #f #f #f))
- (vector-desc	 #("scheme_value ~a" "" #f #f #f))
- (pair-desc	 #("scheme_value ~a" "" #f #f #f))
+ ; Extension by JMG with the new FFI
+ (time_t  #("time_t ~a"		 "s48_enter_integer" #f #f #f))
+ (clock_t #("clock_t ~a"	 "s48_enter_fixnum" #f #f #f))
+ (ssize_t #("ssize_t ~a"	 "s48_enter_fixnum" #f #f #f))
+
+
+ (desc		 #("s48_value ~a" "" #f #f #f))
+ (string-desc	 #("s48_value ~a" "" #f #f #f))
+ (char-desc	 #("s48_value ~a" "" #f #f #f))
+ (integer-desc	 #("s48_value ~a" "" #f #f #f))
+ (vector-desc	 #("s48_value ~a" "" #f #f #f))
+ (pair-desc	 #("s48_value ~a" "" #f #f #f))
 
  (string 	#("const char *~a" #f ,str-and-len->carrier make-string-carrier
 				   string-carrier->string))
@@ -440,7 +458,7 @@
     (format #f (rep:c-decl i) outer-type)))
 
 (define (info->var-decl i var)
-  (format #f "~%    ~a;" ; statement-ize decl.
+  (format #f "~%    ~a = 0;" ; statement-ize decl.
 	  (format #f (rep:c-decl i) var))) ; decl-ize var.
 
 (define (make-gensym prefix i)
@@ -589,24 +607,22 @@
 	   (values (format #f "~%    ret1 = ~a;" (Scheme-ize-exp cvtr c-var))
 		   0)))
 	(else
-	 (values (format #f "~%    ret1 = VECTOR_REF(*args,0);~%    ~a"
+	 (values (format #f "~%    ret1 = S48_VECTOR_REF(mv_vec,0);~%    ~a"
 			 (Scheme-ize->carrier info "ret1" c-var))
 		 1))))
 
 ;;; Minor ret value rep-conversion.
 ;;; Convert and store into minor-value vector at entry j.
 (define (make-minor-retval-stmt c-var info j)
-  (let ((target (format #f "VECTOR_REF(*args,~d)" j)))
-    (values (cond ((retrep:immediate info) =>
-                   (lambda (cvtr)
-		     (format #f "~%    ~a = ~a;"
-			     target (Scheme-ize-exp cvtr c-var))))
-		  (else
+  (values (cond ((retrep:immediate info) =>
+                 (lambda (cvtr)
+		   (format #f "~%    S48_VECTOR_SET(mv_vec,~d,~a);"
+			   j (Scheme-ize-exp cvtr c-var))))
+		(else
+		 (let ((target (format #f "S48_VECTOR_REF(mv_vec,~d)" j)))
 		   (format #f "~%    ~a"
-			   (Scheme-ize->carrier info target c-var))))
-	    (+ j 1))))
-
-
+			   (Scheme-ize->carrier info target c-var)))))
+	  (+ j 1)))
 
 (define (stmts strings) (apply string-append strings))
 
@@ -630,7 +646,6 @@
   (destructure (( (#f scheme-name (c-name . params) . return-reps) form ))
     (let* ((c-name (stringify c-name))
 	   (reps (map car params))
-	   
 	   (no-declare? (and (pair? return-reps)
 			     (eq? 'no-declare (car return-reps))))
 	   (return-reps (if no-declare? (cdr return-reps)
@@ -643,7 +658,10 @@
 	   (ret-infos1 (parse-return-reps return-reps
 					  (lambda (x) x)))
 	   (ignore? (string? ret-infos1))
-		      
+	   (gc-protected-vars (if ignore?  ; maybe extended by mv_vec
+				  '() 
+				  '("ret1")))
+
 	   (ret-infos2 (if (not ignore?)	; Flatten them out.
 			   (apply append ret-infos1))) 
 	   (ret-infos3 (if (not ignore?)	; A canonical representative
@@ -665,7 +683,22 @@
 				;; Is major ret val non-immediate
 				(not (retrep:immediate
 				      (caar ret-infos1))))))
-
+	   (gc-protected-vars (if mv-return? 
+				  (cons "mv_vec" gc-protected-vars)
+				  gc-protected-vars))
+	   (gc_declare (if (> (length gc-protected-vars) 0) 
+			   (format #f "~%    S48_DECLARE_GC_PROTECT(~d);" 
+			       (length gc-protected-vars))
+			   ""))
+	   (gc_protect (if (> (length gc-protected-vars) 0)
+			   (format #f "~%    S48_GC_PROTECT_~d(~a);"
+			       (length gc-protected-vars)
+			       (if (null? (cdr gc-protected-vars))
+				   (car gc-protected-vars)
+				   (string-append (car gc-protected-vars)
+						  ","
+						  (cadr gc-protected-vars))))
+			   ""))
 	   (nargs (length reps))
 	   (stub-nargs (if mv-return? (+ nargs 1) nargs))
 	   (other-retvals (if ignore? '() (cdr ret-infos3)))
@@ -673,9 +706,12 @@
 	   (ret-var-decls (stmts (map info->var-decl 
 				      other-retvals ret-vars)))
 	   
-	   ;; List of the form ("arg[2]" "arg[1]" "arg[0]").
-	   (stub-args (make-stub-args nargs mv-return?))
-	   
+	   ; Frank: begin
+	   (gensym (let ((gs (make-gensym "g" 0)))
+		     (lambda () (gs #f))))
+	   ; the c-stubs formal parameters are named "g1" ... "gn"
+	   (stub-args (map (lambda (p) (gensym)) params))
+	   ; Frank: end
 	   (post-C-vars (make-post-C-var-list params-info))
 	   (pc-var-decls (stmts (map (lambda (i v)
 				       (if v (info->var-decl i v) ""))
@@ -724,47 +760,58 @@
 	   (mv-assigns (if ignore? ""
 			   (make-mv-assigns (cons "r1" ret-vars)
 					    ret-infos1)))
-
+	   (gc_unprotect (if (> (length gc-protected-vars) 0)
+			     "\n    S48_GC_UNPROTECT();"
+			     ""))
 	   (return-stmt (format #f "~%    return ~a;"
-				(if ignore? "SCHFALSE" "ret1")))
+				(if ignore? "S48_FALSE" "ret1")))
 
 	   ;; Do the call, release the free-vars, do the mv-return
 	   ;; assignments, then return.
 	   (epilog (if ignore?
-		       (string-append c-call ";" post-C-val-processing return-stmt)
-		       (string-append "r1 = " c-call ";"
+		       (string-append  c-call ";" post-C-val-processing return-stmt)
+		       (string-append  gc_protect "\n    r1 = " c-call ";"
 				      post-C-val-processing
-				      mv-assigns return-stmt))))
+				      mv-assigns gc_unprotect return-stmt))))
 ;     (breakpoint)
       (format #f cfun-boilerplate
 	      c-name
+	      ; Frank: begin
+	      ; multiple values will be returned in the c-stubs last formal parameter:
+	      ; the vector mv_vec
+	      (let ((args (if mv-return? (append stub-args '("mv_vec")) stub-args)))
+		(proto-args (map (lambda (var) (string-append "s48_value " var)) args)))
+	      ; Frank: end
 	      c-fun-decl
 	      (if ignore? "" ret1-decl)
+	      gc_declare
 	      primary-retvar-decl ret-var-decls pc-var-decls
-	      stub-nargs c-name
 	      pc-var-assigns
 	      epilog))))
 
+; Frank: begin
 (define cfun-boilerplate
-  "scheme_value df_~a(long nargs, scheme_value *args)
+  "s48_value df_~a(~a)
 {
-    ~a~a~a~a~a
+    ~a~a~a~a~a~a
 
-    cig_check_nargs(~d, nargs, \"~a\");~a
     ~a
-    }
+    ~a
+}
 
 ")
+; Frank: end
 
 (define ret1-decl
   "
-    scheme_value ret1;")
+    s48_value ret1 = S48_FALSE;")
 
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 (define cfile-header-boilerplate
   "/* This is an Scheme48/C interface file, 
-** automatically generated by cig.
+** automatically generated by a hacked version of cig 3.0.
+step 4
 */
 
 #include <stdio.h>
@@ -773,60 +820,118 @@
 
 ")
 
+; Frank: begin
+
+(define s48-init-boilerplate
+  "void s48_init_~a(void)
+{~a
+}
+")
+
 (define (define-foreign-process-form form oport)
-  (if (pair? form)
-      (case (car form)
+  ; c-names will be the list of c-names of all define-foreign-forms
+  (define c-names '())
+  (define init-name #f)
+  (define (define-foreign-process-form2 form)
+    (if (pair? form)
+	(case (car form)
+	  
+	  ((begin)
+	   (if (list? (cdr form))
+	       (for-each (lambda (f) (define-foreign-process-form2 f))
+			 (cdr form))))
+	  
+	  ((define-structure define-structures)
+	   (if (and (pair? (cdr form))
+		    (list? (cddr form)))
+	       (let ((clauses (cddr form)))
+		 (for-each (lambda (clause)
+			     (if (and (pair? clause)
+				      (eq? 'begin (car clause)))
+				 (define-foreign-process-form2 clause)))
+			   clauses))))
+	  
+	  ((define-foreign)
+	   (let ((c-name (string-append "df_" (stringify (caaddr form)))))
+	     (begin
+	       (set! c-names (cons c-name c-names))
+	       (display (define-foreign->C-stub form) oport))))
 
-	((begin)
-	 (if (list? (cdr form))
-	     (for-each (lambda (f) (define-foreign-process-form f oport))
-		       (cdr form))))
+	  ((foreign-init-name)
+	   (let ((name (cdr form)))
+	     (set! init-name (car name))))
 
-	((define-structure define-structures)
-	 (if (and (pair? (cdr form))
-		  (list? (cddr form)))
-	     (let ((clauses (cddr form)))
-	       (for-each (lambda (clause)
-			   (if (and (pair? clause)
-				    (eq? 'begin (car clause)))
-			       (define-foreign-process-form clause oport)))
-			 clauses))))
+	  ((define-stubless-foreign)
+	   (let ((c-name (cadddr form)))
+	     (set! c-names (cons c-name c-names))))
+	    
+	  ((foreign-source)
+	   (let ((forms (cdr form)))
+	     (if (pair? forms)
+		 (begin (display (car forms) oport)
+			(map (lambda (x)
+			       (newline oport)
+			       (display x oport))
+			     (cdr forms)))))))))
+  (define-foreign-process-form2 form)
+  (values (reverse c-names) init-name))
 
-	((define-foreign)
-	 (display  (define-foreign->C-stub form) oport))
-
-	((foreign-source)
-	 (let ((forms (cdr form)))
-	   (if (pair? forms)
-	       (begin (display (car forms) oport)
-		      (map (lambda (x)
-			     (newline oport)
-			     (display x oport))
-			   (cdr forms)))))))))
+; Frank: end
+(define (display-register c-names init-name oport)
+  (if (not init-name)
+      (error "no foreign-init-name statement found")
+      (let ((register-txt
+	     (apply
+	      string-append
+	      (map (lambda (c-name)
+		     (format #f "~%    S48_EXPORT_FUNCTION(~a);" c-name))
+		   c-names))))
+	(format oport s48-init-boilerplate init-name register-txt))))
 
 (define (process-define-foreign-stream iport oport)
   (display cfile-header-boilerplate oport)
-  (let lp ()
+  (let lp ((c-names '()) (init-name #f))
     (let ((form (read iport)))
-      (cond ((not (eof-object? form))
-	     (define-foreign-process-form form oport)
-	     (lp))))))
+      (if (eof-object? form)
+	  (display-register c-names init-name oport)
+	  (receive (new-c-names maybe-init-name)
+		   (define-foreign-process-form form oport)
+		   (let ((init-name (if maybe-init-name 
+					(if init-name
+					    (error "multiple foreign-init-name definitions")
+					    maybe-init-name)
+					init-name)))
+		     (lp (append c-names new-c-names) init-name)))))))
 
-(define (process-define-foreign-file fname)
+; Frank: begin
+; (process-define-foreign-file fname) scans file fname.scm and produces a c-stub for every
+; scanned define-foreign form and places it in file fname.c.
+(define (process-define-foreign-file fname init-name)
   (call-with-input-file (string-append fname ".scm")
     (lambda (iport)
       (call-with-output-file (string-append fname ".c")
 	(lambda (oport)
-	  (process-define-foreign-stream iport oport))))))
+	  (begin
+	    (display cfile-header-boilerplate oport)
+	    (let lp ((c-names '()))
+	      (let ((form (read iport)))
+		(if (eof-object? form)
+		    (let ((register-txt
+			   (apply
+			    string-append
+			    (map (lambda (c-name)
+				   (format #f "~%    S48_EXPORT_FUNCTION(~a);" c-name))
+				 c-names))))
+		      (format oport s48-init-boilerplate init-name register-txt))
+		    (lp (append c-names (define-foreign-process-form form oport))))))))))))
+; Frank: end
 
 
-(define (cig-standalone-toplevel . args) ; ignore your args.
+(define (cig-standalone-toplevel f-and-init-name) ; ignore your args no
   (process-define-foreign-stream (current-input-port)
 				 (current-output-port))
   0)
 
-
-
 ;;; This section defines the Scheme-side macro processor.
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; (define-syntax define-foreign define-foreign-expander)
@@ -865,9 +970,12 @@
 	   (%define (rename 'define))
 	   (%let (rename 'let))
 	   (%lambda (rename 'lambda))
-	   (%external-call (rename 'external-call))
-	   (%get-external (rename 'get-external))
-
+	   ; JMG: begin replaced external-lambda by import-lambda-definition
+	   (%import-lambda-definition (rename 'import-lambda-definition))
+	   (%lookup-imported-binding (rename 'lookup-imported-binding))
+	   (%call-imported-binding (rename 'call-imported-binding))
+	   (%let* (rename 'let*))
+	   ; JMG: end
 	   (gensym (let ((gs (make-gensym "g" -1)))
 		     (lambda () (string->symbol (gs #f)))))
 
@@ -894,30 +1002,49 @@
 	   (%f (rename 'f)))
 
       (if (not carrier-vec?)
-	  (let* ((xcall `(,%external-call ,%f ,@c-args))
+	  (let* ((xcall `(,%f ,@c-args))
 		 (xcall (cond ((and (not ignore?)
 				    (retrep:s-cvtr (car ret-infos2)))
 			       => (lambda (proc) `(,proc ,xcall))) ; not hygenic
 			      (else xcall))))
 
-	  `(,%define ,scheme-name
-	     (,%let ((,%f (,%get-external ,c-name)))
-	       (,%lambda ,args ,xcall))))
-
-	  (let ((retarg1 (rename 'r1))
+	    ; Frank: begin
+	    ; get-external and external-call replaced: now external-lambda
+	    ;JMG now import-lambda-definition
+	    `(,%import-lambda-definition ,scheme-name ,args ,c-name))
+	   (let ((retarg1 (rename 'r1))
 		(retarg2 (rename 'r2))
 		(%make-vector (rename 'make-vector)))
 	    `(,%define ,scheme-name
-	       (,%let ((,%f (,%get-external ,c-name)))
+	       (,%let ((,%f (,%lookup-imported-binding ,c-name)))
 	         (,%lambda ,args
 		   (,%let ((,retarg2 (,%make-vector ,carrier-veclen)))
 		     ,@(install-carriers retarg2 ret-infos2
 					 (rename 'vector-set!))
-		     (,%let ((,retarg1 (,%external-call ,%f ,@c-args ,retarg2)))
+		     (,%let ((,retarg1 (,%call-imported-binding ,%f ,@c-args ,retarg2)))
 		       (values ,@(make-values-args retarg1 retarg2
 						   ret-infos2
 						   rename))))))))))))
+;;	  `(,%define ,scheme-name
+;;	     (,%let ((,%f (,%external-lambda ,args ,c-name)))
+;;	       (,%lambda ,args ,xcall))))
 
+;	  (let ((retarg1 (rename 'r1))
+;		(retarg2 (rename 'r2))
+;		(%make-vector (rename 'make-vector)))
+;	    `(,%define 
+;	      ,scheme-name 
+;	      (,%let* ((temp (,%lookup-imported-binding ,c-name))
+;		       (,%f (,%lambda ,args (,%call-imported-binding temp ,args))))
+;		      (,%lambda ,args
+;				(,%let ((,retarg2 (,%make-vector ,carrier-veclen)))
+;				       ,@(install-carriers retarg2 ret-infos2
+;							   (rename 'vector-set!))
+;				       (,%let ((,retarg1 (,%f ,@c-args ,retarg2)))
+;					      (values ,@(make-values-args retarg1 retarg2
+;									  ret-infos2
+;									  rename))))))))))))
+; Frank: end
 (define (install-carriers carrier-vec ret-infos2 %vector-set!)
   ;; Skip the major ret value if it doesn't require a carrier.
   (let* ((major-rep (and (pair? ret-infos2) (car ret-infos2)))
@@ -964,9 +1091,12 @@
 
 
 
-(define-structure define-foreign-syntax (export (define-foreign :syntax)
-						(foreign-source :syntax))
-  (open scheme externals structure-refs cig-aux)
+(define-structure define-foreign-syntax 
+  (export (define-foreign :syntax)
+	  (foreign-source :syntax)
+	  (foreign-init-name :syntax)
+	  (define-stubless-foreign :syntax))
+  (open scheme external-calls  structure-refs cig-aux)
   (access signals) ; for ERROR
   (for-syntax (open scheme define-foreign-syntax-support))
   (begin
@@ -979,6 +1109,15 @@
       (syntax-rules ()
 	((foreign-source stuff ...) #f)))
 
+    (define-syntax foreign-init-name
+      (syntax-rules ()
+	((foreign-init-name name) (lambda () name))))
+
+    (define-syntax define-stubless-foreign
+      (syntax-rules ()
+         ((define-stubless-foreign bla ...)
+	  (import-lambda-definition bla ...))))
+
     (define (check-arg pred obj proc)
       (if (not (pred obj))
 	  (error "check-arg" pred obj proc)
@@ -987,3 +1126,46 @@
 
 
 ;;; Todo: "info" terminology is gone. Clean up.
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
+
