@@ -2,45 +2,14 @@
 ;;; Posix advisory record-locking for file descriptors.
 ;;; These procs may only be applied to integer file descriptors; 
 ;;; they may not be applied to ports.
-;;; Copyright (c) 1995 by David Albertz and Olin Shivers. See file COPYING.
+;;; Copyright (c) 1995 by David Albertz and Olin Shivers.
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
 ;;; C syscall interface
 ;;;;;;;;;;;;;;;;;;;;;;;
 
-(foreign-source
-  "#include <sys/types.h>"
-  "#include <unistd.h>"
-  "#include <errno.h>"
-  "#include <fcntl.h>"
-  ""
-  "/* Make sure foreign-function stubs interface to the C funs correctly: */"
-  "#include \"flock1.h\""
-  ""
-  "#define errno_or_false(x) (((x) == -1) ? ENTER_FIXNUM(errno) : SCHFALSE)"
-  "" "")
-
-(define-foreign %set-lock (set_lock (integer fd)
-				    (integer cmd)
-				    (integer type)
-				    (integer whence)
-				    (integer start)
-				    (integer len))
-  (to-scheme integer errno_or_false))
-
-(define-foreign %get-lock (get_lock (integer fd)
-				    (integer cmd)
-				    (integer type)
-				    (integer whence)
-				    (integer start)
-				    (integer len))
-  (to-scheme integer errno_or_false)
-  integer	; lock type
-  integer	; whence
-  integer	; start
-  integer	; len
-  integer)	; pid
-				       
+(import-os-error-syscall %set-lock (fd cmd type whence start len) "set_lock")
+(import-os-error-syscall %get-lock (fd cmd type whence start len) "get_lock")
 
 ;;; The LOCK record type
 ;;;;;;;;;;;;;;;;;;;;;;;;
@@ -93,40 +62,36 @@
 ;;; The main routines
 ;;;;;;;;;;;;;;;;;;;;;
 
-(define-errno-syscall (lock-region fdes lock)
-  (lambda (fdes lock)
-    (call-lock-region %set-lock fcntl/set-record-lock fdes lock)))
+(define (lock-region fdes lock)
+  (if (not (lock-region/no-block fdes lock))
+      (begin
+	(relinquish-timeslice)
+	(lock-region fdes lock))))
 
 ;;; Return true/false indicating success/failure.
 
 (define (lock-region/no-block fdes lock)
-  (cond ((call-lock-region %set-lock fcntl/set-record-lock-no-block fdes lock) 
-         => (lambda (errno)
-	      (cond ((or (= errno errno/again) (= errno errno/acces)) #f)
-		    ((= errno errno/intr) (lock-region/no-block fdes lock))
-		    (else (errno-error errno lock-region/no-block fdes lock)))))
-	(else #t)))
-
+  (with-errno-handler
+   ((errno data)
+    ((errno/again errno/acces) #f))
+   (call-lock-region %set-lock fcntl/set-record-lock-no-block fdes lock)
+   #t))
 
 ;;; Return first lock that conflicts w/LOCK; if none, return #f.
 
 (define (get-lock-region fdes lock)
-  (receive (err type whence start len pid)
-           (call-lock-region %get-lock fcntl/get-record-lock fdes lock)
-    (cond ((not err)
+  (apply (lambda (type whence start len pid)
 	   (and (not (= type lock/release))
 		(make-%lock-region (= type lock/write) start len whence
 				   (pid->proc pid 'create))))
-	  ((= err errno/intr) (get-lock-region fdes lock))
-	  (else (errno-error err get-lock-region fdes lock)))))
+	 (call-lock-region %get-lock fcntl/get-record-lock fdes lock)))
 
 
-(define-errno-syscall (unlock-region fdes lock)
-  (lambda (fdes lock)
-    (%set-lock fdes fcntl/set-record-lock lock/release
-	       (lock-region:whence lock)
-	       (lock-region:start lock)
-	       (lock-region:len lock))))
+(define (unlock-region fdes lock)
+  (%set-lock fdes fcntl/set-record-lock lock/release
+	     (lock-region:whence lock)
+	     (lock-region:start lock)
+	     (lock-region:len lock)))
 
 
 ;;; Locks with dynamic extent -- with and without sugar
