@@ -1,9 +1,8 @@
 ;;; Field and record parsing utilities for scsh.
-;;; Copyright (c) 1994 by Olin Shivers.
+;;; Copyright (c) 1994 by Olin Shivers. See file COPYING.
 
 ;;; Notes:
 ;;; - Comment on the dependencies here...
-;;; - Redefine READ-LINE using READ-DELIMITED.
 ;;; - Awk should deal with case-insensitivity.
 ;;; - Should I change the field-splitters to return lists? It's the
 ;;;   right thing, and costs nothing in terms of efficiency.
@@ -55,43 +54,12 @@
 ;;; This has the effect you want with field parsing. For example, if you split
 ;;; a string with the empty pattern, you will explode the string into its
 ;;; individual characters:
-;;;     ((suffix-splitter "") "foo") -> #("" "f" "o" "o")
+;;;     ((suffix-splitter (rx "")) "foo") -> #("" "f" "o" "o")
 ;;; However, even though this boundary case is handled correctly, we don't
 ;;; recommend using it. Say what you mean -- just use a field splitter:
-;;;     ((field-splitter ".") "foo") -> #("f" "o" "o")
+;;;     ((field-splitter (rx any)) "foo") -> #("f" "o" "o")
 
 
-
-;;; (join-strings string-list [delimiter grammar]) => string
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;; Paste strings together using the delimiter string.
-;;;
-;;; (join-strings '("foo" "bar" "baz") ":") => "foo:bar:baz"
-;;;
-;;; DELIMITER defaults to a single space " "
-;;; GRAMMAR is one of the symbols {infix, suffix} and defaults to 'infix.
-
-;;; (join-strings strings [delim grammar])
-
-(define (join-strings strings . args)
-  (if (pair? strings)
-      (receive (delim grammar) (parse-optionals args " " 'infix)
-	(check-arg string? delim join-strings)
-	(let ((strings (reverse strings)))
-	  (let lp ((strings (cdr strings))
-		   (ans (case grammar
-			  ((infix)  (list (car strings)))
-			  ((suffix) (list (car strings) delim))
-			  (else (error "Illegal grammar" grammar)))))
-	    (if (pair? strings)
-		(lp (cdr strings)
-		    (cons (car strings) (cons delim ans)))
-	  
-		; All done
-		(apply string-append ans)))))
-
-      ""))	; Special-cased for infix grammar.
-
 ;;; FIELD PARSERS
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; This section defines routines to split a string into fields.
@@ -101,13 +69,13 @@
 
 (define (->delim-matcher x)
   (if (procedure? x) x					; matcher proc
-      (let ((re (cond ((regexp? x) x)			; regexp pattern
-		      ((string? x) (make-regexp x))	; regexp string
-		      (else (error "Illegal pattern/parser" x)))))
-
-	;; The matcher proc.
+      (let ((re (cond ((string? x) (re-string x))
+		      ((char-set? x) (re-char-set x))
+		      ((char? x) (re-string (string x)))
+		      ((regexp? x) x)
+		      (else (error "Illegal field-reader delimiter value" x)))))
 	(lambda (s i)
-	  (cond ((regexp-exec re s i) =>
+	  (cond ((regexp-search re s i) =>
 		 (lambda (m) (values (match:start m 0) (match:end m 0))))
 		(else (values #f #f)))))))
 
@@ -121,9 +89,9 @@
 (define (make-field-parser-generator default-delim-matcher loop-proc)
   ;; This is the parser-generator
   (lambda args
-    (receive (delim-spec num-fields handle-delim)
-	     (parse-optionals args  default-delim-matcher #f 'trim)
-
+    (let-optionals args ((delim-spec default-delim-matcher)
+			 (num-fields #f)
+			 (handle-delim 'trim))
       ;; Process and error-check the args
       (let ((match-delim (->delim-matcher delim-spec))
 	    (cons-field (case handle-delim	 	; Field     is s[i,j).
@@ -151,17 +119,18 @@
 
 	  ;; This is the parser.
 	  (lambda (s . maybe-start)
-	    (reverse (loop-proc s (optional-arg maybe-start 0)
+	    (reverse (loop-proc s (:optional maybe-start 0)
 				match-delim cons-field
 				num-fields nfields-exact?))))))))
 
-(define default-field-matcher (->delim-matcher "[^ \t\n]+"))
+;;; Default field spec is runs of non-whitespace chars.
+(define default-field-matcher (->delim-matcher (rx (+ (~ white)))))
 
 ;;; (field-splitter [field-spec num-fields])
 
 (define (field-splitter . args)
-  (receive (field-spec num-fields)
-	   (parse-optionals args  default-field-matcher #f)
+  (let-optionals args ((field-spec default-field-matcher)
+		       (num-fields #f))
 
     ;; Process and error-check the args
     (let ((match-field (->delim-matcher field-spec)))
@@ -175,7 +144,7 @@
 
 	;; This is the parser procedure.
 	(lambda (s . maybe-start)
-	  (reverse (fieldspec-field-loop s (optional-arg maybe-start 0)
+	  (reverse (fieldspec-field-loop s (:optional maybe-start 0)
 					 match-field num-fields nfields-exact?)))))))
 
 
@@ -314,8 +283,8 @@
 
 ;;; Now, build the exported procedures: {infix,suffix,sloppy-suffix}-splitter.
 
-(define default-suffix-matcher (->delim-matcher "[ \t\n]+|$"))
-(define default-infix-matcher  (->delim-matcher "[ \t\n]+"))
+(define default-suffix-matcher (->delim-matcher (rx (| (+ white) eos))))
+(define default-infix-matcher  (->delim-matcher (rx (+ white))))
 
 (define infix-splitter
   (make-field-parser-generator default-infix-matcher  infix-field-loop))
@@ -326,82 +295,6 @@
 
 
 
-;;; Delimited readers
-;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;; We repeatedly allocate a buffer and fill it with READ-DELIMITED!
-;;; until we hit a delimiter or EOF. Each time through the loop, we
-;;; double the total buffer space, so the loop terminates with a log
-;;; number of reads, but uses at most double the optimal buffer space.
-
-(define (read-delimited delims . maybe-port)
-  (let ((smart-substring (lambda (s end)
-			   (if (= end (string-length s)) s
-			       (substring s 0 end))))
-	(delims (->char-set delims)))
-
-    ;; BUFLEN is total amount of buffer space allocated to date.
-    (let lp ((strs '()) (buflen 80) (buf (make-string 80)))
-      (cond ((apply read-delimited! delims buf maybe-port) =>
-             (lambda (i)
-	       (if (null? strs)		; Gratuitous optimisation.
-		   (smart-substring buf i)
-		   (apply string-append
-			  (reverse (if (eof-object? i)
-				       strs
-				       (cons (smart-substring buf i)
-					     strs)))))))
-
-	    (else (lp (cons buf strs)
-		      (+ buflen buflen)
-		      (make-string buflen)))))))
-
-
-;;; (read-delimited! delims buf [port start end])
-
-(define (read-delimited! delims buf . args) ; [port start end]
-  (receive (port start end)
-	   (parse-optionals args (current-input-port) 0 (string-length buf))
-    (check-arg input-port? port read-delimited!)
-    (let ((delims (->char-set delims)))
-;      (if (fd-inport? port) ; ???
-;
-;	  ;; Handle fdports in C code for speed.
-;	  (receive (err val)
-;		   (%read-delimited-fdport!/errno delims buf port start end)
-;	    (if err
-;	        (errno-error err read-delimited!)
-;	        val))
-
-	  ;; This is the code for other kinds of ports.
-	  (let lp ((i start))
-	    (and (< i end)
-	         (let ((c (peek-char port)))
-		   (if (or (eof-object? c)
-			   (char-set-member? delims c))
-		       (- i start)
-		       (begin (string-set! buf i (read-char port))
-			      (lp (+ i 1))))))))))
-;)
-
-;(define-foreign %read-delimited-fdport!/errno (read_delim (string-desc delims)
-;							  (string-desc buf)
-;							  (desc port) ;???
-;							  (fixnum start)
-;							  (fixnum end))
-;  desc	; errno or #f
-;  desc)	; nread or #f or eof-object
-
-(define (skip-char-set cset . maybe-port)
-  (let ((port (optional-arg maybe-port (current-input-port))))
-    (let lp ()
-      (let ((c (peek-char port)))
-	(cond ((and (char? c) (char-set-member? cset c))
-	       (read-char port)
-	       (lp))
-	      (else c))))))
-
-
-
 ;;; Reading records
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 
@@ -411,73 +304,49 @@
 ;;; (reader [port]) -> string or eof
 
 (define (record-reader . args)
-  (receive (delims elide? handle-delim)
-           (parse-optionals args default-record-delims #f 'trim)
+  (let-optionals args ((delims default-record-delims)
+		      (elide? #f)
+		      (handle-delim 'trim))
     (let ((delims (->char-set delims)))
 
       (case handle-delim
 	((trim)			; TRIM-delimiter reader.
 	 (lambda maybe-port
 	   (let ((s (apply read-delimited delims maybe-port)))
-	     (if (not (eof-object? s))
-		 (if elide?
-		     (apply skip-char-set delims maybe-port) ; Snarf delims.
-		     (apply read-char maybe-port))) ; Just snarf one.
+	     (if (and (not (eof-object? s)) elide?)
+		 (apply skip-char-set delims maybe-port)) ; Snarf extra delims.
 	     s)))
 
-	((concat split)		; CONCAT-delimiter & SPLIT-delimiter reader.
-	 (let ((not-delims (char-set-invert delims)))
+	((concat)		; CONCAT-delimiter reader.
+	 (let ((not-delims (char-set-complement delims)))
 	   (lambda maybe-port
-	     (let ((s (apply read-delimited delims maybe-port)))
-	       (if (eof-object? s) s
-		   (let ((delim (if elide?
-				    (apply read-delimited not-delims maybe-port)
-				    (string (apply read-char maybe-port)))))
-		     (if (eq? handle-delim 'split)
-			 (values s delim)
-			 (if (eof-object? delim) s
-			     (string-append s delim)))))))))
+	     (let* ((p (:optional maybe-port (current-input-port)))
+		    (s (read-delimited delims p 'concat)))
+	       (if (or (not elide?) (eof-object? s)) s
+		   (let ((extra-delims (read-delimited not-delims p 'peek)))
+		     (if (eof-object? extra-delims) s
+			 (string-append s extra-delims))))))))
+
+	((split)		; SPLIT-delimiter reader.
+	 (let ((not-delims (char-set-complement delims)))
+	   (lambda maybe-port
+	     (let ((p (:optional maybe-port (current-input-port))))
+	       (receive (s delim) (read-delimited delims p 'split)
+		 (if (eof-object? s) (values s s)
+		     (values s
+			     (if (or (not elide?) (eof-object? delim))
+				 delim
+				 ;; Elide: slurp in extra delims.
+				 (let ((delim (string delim))
+				       (extras (read-delimited not-delims
+							       p 'peek)))
+				   (if (eof-object? extras) delim
+				       (string-append delim extras)))))))))))
 
 	(else
 	 (error "Illegal delimiter-action" handle-delim))))))
 
 
-;;; {string, char, char-set, char predicate} -> char-set
-
-(define (->char-set x)
-  (cond ((char-set? x) x)
-	((string? x) (string->char-set x))
-	((char? x) (char-set x))
-	((procedure? x) (predicate->char-set x))
-	(else (error "->char-set: Not a charset, string, char, or predicate."
-		     x))))
-
-
-
-(define blank-line-regexp (make-regexp "^[ \t]*\n$"))
-
-;;; (read-paragraph [port])
-(define (read-paragraph . maybe-port)
-  (let ((port (optional-arg maybe-port (current-input-port))))
-    
-    ;; First, skip all blank lines.
-    (let lp ()
-      (let ((line (read-line port #t)))
-	(cond ((eof-object? line)                   line)
-	      ((regexp-exec blank-line-regexp line) (lp))
-
-	      ;; Then, read in non-blank lines.
-	      (else (let ((lines (let lp ((lines (list line)))
-				   (let ((line (read-line port #t)))
-				     (cond ((or (eof-object? line)
-						(regexp-exec blank-line-regexp
-							     line))
-					    lines)
-					   (else (lp (cons line lines))))))))
-
-		      ;; Return the paragraph
-		      (apply string-append (reverse lines)))))))))
-
 ;;; Reading and parsing records
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
 ;;; (field-reader [field-parser rec-reader]) -> reader
@@ -489,8 +358,8 @@
 (define default-field-parser (field-splitter))
 
 (define (field-reader . args)
-  (receive (parser rec-reader)
-           (parse-optionals args default-field-parser read-line)
+  (let-optionals args ((parser    default-field-parser)
+		       (rec-reader read-line))
     (lambda maybe-port
       (let ((record (apply rec-reader maybe-port)))
 	(if (eof-object? record)
@@ -509,60 +378,17 @@
 ;;; Repeatedly do (APPLY PROC M STATE) to generate new state values,
 ;;; where M is a regexp match structure made from matching against STRING.
 
-;(define (regexp-reduce string start regexp proc . state)
-;  (let ((end (string-length string))
-;	(regexp (if (string? regexp)
-;		    (make-regexp regexp)
-;		    regexp)))
-;
+;(define (regexp-fold string start regexp proc . state)
+;  (let ((end (string-length string)))
 ;    (let lp ((i start) (state state) (last-null? #f))
 ;      (let ((j (if last-null? (+ i 1) i)))
-;	(cond ((and (<= j end) (regexp-exec regexp string j)) =>
+;	(cond ((and (<= j end) (regexp-search regexp string j)) =>
 ;               (lambda (m)
 ;		 (receive state (apply proc m state)
 ;		   (lp (match:end m) state (= (match:start m) (match:end m))))))
 ;	      (else (apply values state)))))))
 ;
 ;(define (all-regexp-matches regexp string)
-;  (reverse (regexp-reduce string 0 regexp
-;			  (lambda (m ans) (cons (match:substring m 0) ans))
-;			  '())))
-
-;;; Previously in newports.scm
-
-;;; Read in a line of data. Input is terminated by either a newline or EOF.
-;;; The newline is trimmed from the string.
-
-(define (read-line . rest)
-  (let ((port (if (null? rest) (current-input-port) (car rest))) ; Optional arg
-	(retain-newline? (and (not (null? rest))		 ; parsing.
-			      (not (null? (cdr rest)))
-			      (cadr rest)))
-
-	;; S[I] := C. If this overflows S, grow it.
-	(deposit (lambda (s i c)
-		   (let ((s (if (< i (string-length s)) s
-				(string-append s s)))) ; doubling hack
-		     (string-set! s i c)
-		     s)))
-
-	;; Precisely resize S to size NUMCHARS.
-	(trim (lambda (s numchars)
-		(if (= numchars (string-length s)) s
-		    (substring s 0 numchars)))))
-
-    (let lp ((s (make-string 81)) (numchars 0))
-      (let ((c (read-char port)))
-	(cond ((eof-object? c)
-	       (if (zero? numchars) c
-		   (trim s numchars)))
-
-	      ((char=? c #\newline)
-	       (if retain-newline?
-		   (trim (deposit s numchars c)
-			 (+ numchars 1))
-		   (trim s numchars)))
-
-	      (else (lp (deposit s numchars c)
-			(+ numchars 1))))))))
-
+;  (reverse (regexp-fold string 0 regexp
+;			 (lambda (m ans) (cons (match:substring m 0) ans))
+;			 '())))
