@@ -5,21 +5,6 @@
 
 ;;; Scheme48 implementation.
 
-(foreign-source
- "#include <sys/types.h>"
- "#include <sys/socket.h>"
- "#include <errno.h>"
- ""
- "/* Make sure foreign-function stubs interface to the C funs correctly: */"
- "#include \"network1.h\""
- ""
- "extern int h_errno;"
- ""
- "#define errno_on_zero_or_false(x) ((x) ? SCHFALSE : ENTER_FIXNUM(errno))"
- "#define errno_or_false(x) (((x) == -1) ? ENTER_FIXNUM(errno) : SCHFALSE)"
- "#define False_on_zero(x) ((x) ? ENTER_FIXNUM(x) : SCHFALSE)"
- "" )
-
 ;;;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 ;;; High Level Prototypes
 ;;;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
@@ -32,7 +17,7 @@
 		      (let* ((host (car  args))
 			     (port (cadr args))
 			     (host (car (host-info:addresses 
-					 (name->host-info host))))
+					  (name->host-info host))))
 			     (port (cond ((integer? port) port)
 					 ((string? port)
 					  (service-info:port 
@@ -54,35 +39,52 @@
       (dynamic-wind
        (lambda () #f)
        (lambda () (connect-socket sock addr) (set! connected #t))
-       (lambda ()
+       (lambda () 
          (if (not connected)
-             (close-socket sock))))
+             (close-socket sock))
+	 ))
       (if connected
           sock
           #f))))
 
 (define (bind-listen-accept-loop protocol-family proc arg)
-  (let ((sock (create-socket protocol-family socket-type/stream))
-	(addr (cond ((= protocol-family protocol-family/internet)
-		     (internet-address->socket-address internet-address/any
-		       (cond ((integer? arg) arg)
-			     ((string? arg)
-			      (service-info:port (service-info arg "tcp")))
-			     (else (error "socket-connect: bad arg ~s" arg)))))
-
-		    ((= protocol-family protocol-family/unix)
-		     (unix-address->socket-address arg))
-
-		    (else
-		     (error "bind-listen-accept-loop: unsupported protocol-family ~s"
-			    protocol-family)))))
-
+  (let* ((sock (create-socket protocol-family socket-type/stream))
+	 (addr (cond ((= protocol-family
+			 protocol-family/internet)
+		      (let ((port (cond ((integer? arg) arg)
+					((string? arg)
+					 (service-info:port 
+					  (service-info arg "tcp")))
+					(else
+					 (error "socket-connect: bad arg ~s"
+						arg)))))
+			(internet-address->socket-address internet-address/any
+							  arg)))
+		     ((= protocol-family
+			 protocol-family/unix)
+		      (unix-address->socket-address arg))
+		     (else 
+		      (error "bind-listen-accept-loop: unsupported protocol-family ~s"
+			     protocol-family)))))
     (set-socket-option sock level/socket socket/reuse-address #t)
     (bind-socket sock addr)
     (listen-socket sock 5)
-    (let loop ()
-      (call-with-values (lambda () (accept-connection sock)) proc)
-      (loop))))
+    (with-handler
+     (lambda (condition more)
+       (with-handler
+	(lambda (condition ignore) (more))
+	(lambda () (close-socket sock)))
+       (more))
+     (lambda ()
+       (let loop ()
+	 (with-errno-handler
+	  ;; ECONNABORTED we just ignore
+	  ((errno packet)
+	   ((errno/connaborted) (loop)))
+	  (call-with-values 
+	   (lambda () (accept-connection sock))
+	   proc)
+	  (loop)))))))
 
 ;;;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 ;;; Socket Record Structure
@@ -99,7 +101,7 @@
 ;;; returns the fdes of a socket
 ;;; not exported
 (define (socket->fdes sock)
-  (fdport-data:fd (extensible-input-port-local-data (socket:inport sock))))
+  (fdport-data:fd (port-data (socket:inport sock))))
 
 ;;;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 ;;; Socket Address Routines
@@ -113,8 +115,7 @@
 		port16))
 	(else 
 	 (make-socket-address address-family/internet
-			      (string-append (integer->string address32) 
-					     (integer->string port16))))))
+			      (cons address32 port16)))))
   
 (define (socket-address->internet-address sockaddr)
   (if (or (not (socket-address? sockaddr))
@@ -122,10 +123,8 @@
 		  address-family/internet)))
       (error "socket-address->internet-address: internet socket expected ~s"
 	     sockaddr)
-      (values (string->integer (substring (socket-address:address sockaddr) 
-					  0 4))
-	      (string->integer (substring (socket-address:address sockaddr)
-					  4 8)))))
+      (values (car (socket-address:address sockaddr))
+	      (cdr (socket-address:address sockaddr)))))
 
 (define (unix-address->socket-address path)
   (if (> (string-length path) 108)
@@ -158,17 +157,30 @@
 	(let* ((fd  (%socket pf type protocol))
 	       (in  (make-input-fdport fd 0))
 	       (out (dup->outport in)))
+	  (set-fdes-status in open/non-blocking)
+	  (set-fdes-status out open/non-blocking)
 	  (make-socket pf in out)))))
 
-(define-foreign %socket/errno
-  (socket (integer pf)
-	  (integer type)
-	  (integer protocol))
-  (multi-rep (to-scheme integer errno_or_false)
-             integer))
 
-(define-errno-syscall (%socket pf type protocol) %socket/errno
-  sockfd)
+;;; Turn a file descriptor into a socket. 
+
+(define (port->socket port pf)
+  ;;; ensure underlying fd is a socket by a random getsockopt call
+  (if (not (port? port))
+      (error "first argument to port->socket is not a port" port))
+  (sleazy-call/fdes
+   port
+   (lambda (fd)
+     (%getsockopt fd level/socket socket/debug)))
+  (let ((in (if (input-port? port) 
+		port
+		(dup->inport port)))
+	(out (if (output-port? port)
+		 port
+		 (dup->outport port))))
+    (make-socket pf in out)))
+
+(import-os-error-syscall %socket (pf type protocol) "scsh_socket")
 
 ;;;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 ;;; close syscall
@@ -187,21 +199,19 @@
 	 (error "bind-socket: socket-address expected ~s" name))
 	(else
 	 (let ((family (socket:family sock)))
-	   (if (not (= family (socket-address:family name)))
-	       (error 
-		"bind-socket: trying to bind incompatible address to socket ~s"
-		name)
-	       (%bind (socket->fdes sock)
-		      family
-		      (socket-address:address name)))))))
+	   (cond ((not (= family (socket-address:family name)))
+		  (error 
+		   "bind-socket: trying to bind incompatible address to socket ~s"
+		   name))
+		 ((and (= family address-family/unix)
+		       (> (string-length name) 107))
+		  (error "bind-socket: path too long" name))
+		 (else
+		  (%bind (socket->fdes sock)
+			 family
+			 (socket-address:address name))))))))
 
-(define-foreign %bind/errno
-  (scheme_bind (integer     sockfd)	; socket fdes
-	       (integer     family)	; address family
-	       (string-desc name))	; scheme descriptor
-  (to-scheme integer errno_or_false))
-
-(define-errno-syscall (%bind sockfd family name) %bind/errno)
+(import-os-error-syscall %bind (sockfd family name) "scheme_bind")
 
 ;;;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 ;;; connect syscall
@@ -215,20 +225,32 @@
 	 (let ((family (socket:family sock)))
 	   (cond ((not (= family (socket-address:family name)))
 		  (error 
-	   "connect: trying to connect socket to incompatible address ~s"
-	   name))
+		   "connect: trying to connect socket to incompatible address ~s"
+		   name))
 		 (else
-		  (%connect (socket->fdes sock)
-			    (socket:family sock)
-			    (socket-address:address name))))))))
+		  (let loop ()
+		    ((structure-ref interrupts disable-interrupts!))
+		    (let ((res (%connect (socket->fdes sock)
+					 (socket:family sock)
+					 (socket-address:address name))))
+		      (cond ((eq? res #t)
+			     ((structure-ref interrupts enable-interrupts!)))
+			    (else (wait-for-channel 
+				   (fdport-data:channel 
+				    (fdport-data (socket:inport sock))))
+				  (if (eq? res 0)
+				      (handle-EINPROGRESS sock)
+				      (loop))))))))))))
 
-(define-foreign %connect/errno
-  (scheme_connect (integer sockfd)	; socket fdes
-		  (integer family)	; address family
-		  (desc    name))	; scheme descriptor
-  (to-scheme integer errno_or_false))
+;;; If connect returned EINPROGRESS, we can check it's success after 
+;;; the next success with getsockopt
 
-(define-errno-syscall (%connect sockfd family name) %connect/errno)
+(define (handle-EINPROGRESS sock)
+  (let ((val (socket-option sock level/socket socket/error)))
+    (if (not (zero? val))
+	(errno-error val "scheme_connect"))))
+
+(import-os-error-syscall %connect (sockfd family name) "scheme_connect")
 
 ;;;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 ;;; listen syscall
@@ -240,14 +262,8 @@
 	 (error "listen-socket: integer expected ~s" backlog))
 	(else
 	 (%listen (socket->fdes sock) backlog))))
-	 
-(define-foreign %listen/errno
-  (listen (integer sockfd)	; socket fdes
-	  (integer backlog))	; backlog
-	no-declare ; for Linux
-  (to-scheme integer errno_or_false))
 
-(define-errno-syscall (%listen sockfd backlog) %listen/errno)
+(import-os-error-syscall %listen (sockfd backlog) "scsh_listen")
 
 ;;;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 ;;; accept syscall
@@ -255,23 +271,26 @@
 (define (accept-connection sock)
   (if (not (socket? sock))
       (error "accept-connection: socket expected ~s" sock)
-      (let* ((family (socket:family sock))
-	     (name   (make-addr family))
-	     (fd     (%accept (socket->fdes sock) family name))
-	     (in     (make-input-fdport fd 0))
-	     (out    (dup->outport in)))
-	(values (make-socket family in out)
-		(make-socket-address family name)))))
+      (let ((family (socket:family sock)))
+	(let loop ()
+	  ((structure-ref interrupts disable-interrupts!))
+	  (let ((fd-addr (%accept (socket->fdes sock) family)))
+	    (cond ((pair? fd-addr)
+		   (let ((fd (car fd-addr))
+			 (addr (cdr fd-addr)))
+		     ((structure-ref interrupts 
+				     enable-interrupts!)) 
+		     (let* ((in     (make-input-fdport fd 0))
+			    (out    (dup->outport in)))
+		       (values (make-socket family in out)
+			       (make-socket-address family addr)))))
+		  (else (wait-for-channel 
+			 (fdport-data:channel 
+			  (fdport-data (socket:inport sock)))) 
+			(loop))))))))
 
-(define-foreign %accept/errno
-  (scheme_accept (integer     sockfd)
-		 (integer     family)
-		 (string-desc name))
-  (multi-rep (to-scheme integer errno_or_false)
-             integer))
+(import-os-error-syscall %accept (sockfd family) "scheme_accept")
 
-(define-errno-syscall (%accept sock family name) %accept/errno
-  sockfd)
 
 ;;;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 ;;; getpeername syscall
@@ -281,19 +300,11 @@
 	  (not (= (socket:family sock) address-family/internet)))
       (error "socket-remote-address: internet socket expected ~s" sock)
       (let* ((family (socket:family sock))
-	     (name   (make-addr family)))
-	(%peer-name (socket->fdes sock)
-		    family
-		    name)
-	(make-socket-address family name))))
+	     (addr (%peer-name (socket->fdes sock)
+			       family)))
+	(make-socket-address family addr))))
 
-(define-foreign %peer-name/errno
-  (scheme_peer_name (integer     sockfd)
-		    (integer     family)
-		    (string-desc name))
-  (to-scheme integer errno_or_false))
-
-(define-errno-syscall (%peer-name sock family name) %peer-name/errno)
+(import-os-error-syscall %peer-name  (sockfd family) "scheme_peer_name")
 
 ;;;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 ;;; getsockname syscall
@@ -303,20 +314,10 @@
 	  (not (= (socket:family sock) address-family/internet)))
       (error "socket-local-address: internet socket expected ~s" sock)
       (let* ((family (socket:family sock))
-	     (name   (make-addr family)))
-	(%socket-name (socket->fdes sock)
-		      family
-		      name)
-	(make-socket-address family name))))
+	     (addr (%socket-name (socket->fdes sock) family)))
+	(make-socket-address family addr))))
 
-(define-foreign %socket-name/errno
-  (scheme_socket_name (integer     sockfd)
-		      (integer     family)
-		      (string-desc name))
-  (to-scheme integer "False_on_zero"))
-
-(define-errno-syscall 
-  (%socket-name sock family name) %socket-name/errno)
+(import-os-error-syscall %socket-name (sockfd family) "scheme_socket_name")
 
 ;;;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 ;;; shutdown syscall
@@ -329,13 +330,7 @@
 	(else
 	 (%shutdown (socket->fdes sock) how))))
 
-(define-foreign %shutdown/errno
-  (shutdown (integer sockfd)
-	    (integer how))
-  (to-scheme integer errno_or_false))
-
-(define-errno-syscall 
-  (%shutdown sock how) %shutdown/errno)
+(import-os-error-syscall %shutdown (sockfd how) "scsh_shutdown")
 
 ;;;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 ;;; socketpair syscall
@@ -343,26 +338,18 @@
 (define (create-socket-pair type)
   (if (not (integer? type))
       (error "create-socket-pair: integer argument expected ~s" type)
-      (receive (s1 s2)
-	       (%socket-pair type)
+      (apply 
+       (lambda (s1 s2)
         (let* ((in1  (make-input-fdport s1 0))
 	       (out1 (dup->outport in1))
 	       (in2  (make-input-fdport s2 0))
 	       (out2 (dup->outport in2)))
 	  (values (make-socket protocol-family/unix in1 out1)
-		  (make-socket protocol-family/unix in2 out2))))))
+		  (make-socket protocol-family/unix in2 out2))))
+       (%socket-pair type))))
 
-;; based on pipe in syscalls.scm
-(define-foreign %socket-pair/errno
-  (scheme_socket_pair (integer type))
-  (to-scheme integer errno_or_false)
-  integer
-  integer)
+(import-os-error-syscall %socket-pair (type) "scheme_socket_pair")
 
-(define-errno-syscall 
-  (%socket-pair type) %socket-pair/errno
-  sockfd1
-  sockfd2)
 
 ;;;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 ;;; recv syscall
@@ -396,32 +383,28 @@
 	       (error "receive-message!: integer expected ~s ~s ~s"
 		      flags start end))
 	      (else 
-	       (generic-receive-message! (socket->fdes socket) flags
+	       (generic-receive-message! socket flags
 					 s start end 
-					 recv-substring!/errno
+					 recv-substring!
 					 (socket:family socket)))))))
 
-(define (generic-receive-message! sockfd flags s start end reader from)
+(define (generic-receive-message! socket flags s start end reader family)
   (if (bogus-substring-spec? s start end)
       (error "Bad substring indices" 
-	     reader sockfd flags
-	     s start end from))
-  (let ((addr (make-addr from)))
-    (let loop ((i start))
-      (if (>= i end) (- i start)
-	  (receive (err nread) 
-		   (reader sockfd flags s i end addr)
-	     (cond (err (if (= err errno/intr) (loop i)
-		         ;; Give info on partially-read data in error packet.
-			    (errno-error err reader sockfd flags
-					 s start i end addr)))
-
-		   ((zero? nread)	; EOF
-		    (values
-		     (let ((result (- i start)))
-		       (and (not (zero? result)) result))
-		     from))
-		   (else (loop (+ i nread)))))))))
+	     reader socket flags
+	     s start end family))
+    (let loop ((i start) (remote #f))
+      (if (>= i end) 
+	  (values (- i start) (make-socket-address family remote))
+	  (let* ((res  (reader socket flags s i end)))
+	  (apply (lambda (nread from)
+		   (cond  ((zero? nread)	; EOF
+			   (values
+			    (let ((result (- i start)))
+			      (and (not (zero? result)) result))
+			    (make-socket-address family from)))
+			  (else (loop (+ i nread) from))))
+		 res)))))
 
 (define (receive-message/partial socket len . maybe-flags)
   (let ((flags (:optional maybe-flags 0)))
@@ -451,46 +434,41 @@
 	       (error "receive-message!/partial: integer expected ~s"
 		      flags))
 	      (else 
-	       (generic-receive-message!/partial (socket->fdes socket)
+	       (generic-receive-message!/partial socket
 						 flags 
 						 s start end
-						 recv-substring!/errno
+						 recv-substring!
 						 (socket:family socket)))))))
 
-(define (generic-receive-message!/partial sockfd flags s start end reader from)
+(define (generic-receive-message!/partial socket flags s start end reader from)
   (if (bogus-substring-spec? s start end)
       (error "Bad substring indices" reader s start end))
 
   (if (= start end) 0 ; Vacuous request.
-      (let ((addr (make-addr from)))
 	(let loop ()
-	  (receive (err nread) 
-		   (reader sockfd flags s start end addr)
-
-	     (cond ((not err)
+	  (apply (lambda (nread addr)
 		    (values (and (not (zero? nread)) nread)
-			    (make-socket-address from addr)))
+		    (make-socket-address from addr)))
+	   (reader socket flags s start end)))))
+		
 
-		   ((= err errno/intr) (loop))
-
-		   ; No forward-progess here.
-		   ((or (= err errno/wouldblock)
-			(= err errno/again))
-		    0) 
-
-		   (else (errno-error err reader sockfd flags
-				      s start start end addr))))))))
-
-(define-foreign recv-substring!/errno
-  (recv_substring (integer sockfd)
-		  (integer flags)
-		  (string-desc buf)
-		  (integer start)
-		  (integer end)
-		  (string-desc name))
-  (multi-rep (to-scheme integer errno_or_false)
-	     integer))
+(define (recv-substring! socket flags buf start end)
+	(let loop ()
+	  ((structure-ref interrupts disable-interrupts!))
+	  (let ((maybe-size-addr 
+		 (%recv-substring! (socket->fdes socket) 
+					 flags buf start end)))
+	    (cond (maybe-size-addr 
+		   ((structure-ref interrupts 
+				   enable-interrupts!)) 
+		   maybe-size-addr)
+		  (else (wait-for-channel 
+			 (fdport-data:channel 
+			  (fdport-data (socket:inport socket)))) 
+			(loop))))))
 
+(import-os-error-syscall %recv-substring! (sockfd flags buf start end)
+  "recv_substring")
 ;;;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 ;;; send syscall
 ;;;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
@@ -504,26 +482,21 @@
 	  ((not (string? s))
 	   (error "send-message: string expected ~s" s))
 	  (else 
-	   (generic-send-message (socket->fdes socket) flags
+	   (generic-send-message socket flags
 				 s start end
-				 send-substring/errno 
+				 send-substring 
 				 (if addr (socket-address:family addr) 0)
-				 (and addr (socket-address:address addr)))))))
+				 (if addr (socket-address:address addr) #f))))))
 
-(define (generic-send-message sockfd flags s start end writer family addr)
+(define (generic-send-message socket flags s start end writer family addr)
   (if (bogus-substring-spec? s start end)
       (error "Bad substring indices" 
-	     sockfd flags family addr
+	     socket flags family addr
 	     s start end writer))
-  (let ((addr (if addr (make-addr family) "")))
+
     (let loop ((i start))
       (if (< i end)
-	  (receive (err nwritten) 
-		   (writer sockfd flags s i end family addr)
-	    (cond ((not err) (loop (+ i nwritten)))
-		  ((= err errno/intr) (loop i))
-		  (else (errno-error err sockfd flags family addr
-				     s start i end writer))))))))
+	  (loop (+ i (writer socket flags s i end family addr))))))
 
 
 (define (send-message/partial socket s . args)
@@ -535,38 +508,41 @@
 	  ((not (string? s))
 	   (error "send-message/partial: string expected ~s" s))
 	  (else 
-           (generic-send-message/partial (socket->fdes socket) flags
+           (generic-send-message/partial socket flags
 					 s start end
-					 send-substring/errno
+					 send-substring
 					 (if addr (socket-address:family addr) 0)
-					 (if addr (socket-address:address addr)))))))
+					 (if addr 
+					     (socket-address:address addr) 
+					     #f))))))
 
-(define (generic-send-message/partial sockfd flags s start end writer family addr)
+(define (generic-send-message/partial socket flags s start end writer family 
+				      addr)
   (if (bogus-substring-spec? s start end)
       (error "Bad substring indices" 
-	     sockfd flags family addr
+	     socket flags family addr
 	     s start end writer))
+  (if (= start end) 
+      0			; Vacuous request.
+      (writer socket flags s start end family addr)))
 
-  (if (= start end) 0			; Vacuous request.
-      (let loop ()
-	(receive (err nwritten) 
-		 (writer sockfd flags s start end family addr)
-		 (cond ((not err) nwritten)
-		       ((= err errno/intr) (loop))
-		       ((or (= err errno/again) (= err errno/wouldblock)) 0)
-		       (else (errno-error err sockfd flags family addr
-					  s start start end writer)))))))
+(define (send-substring socket flags buf start end family name)
+  (let loop ()
+    ((structure-ref interrupts disable-interrupts!))
+    (cond ((%send-substring (socket->fdes socket) flags buf start end 
+			    family name)
+	   => (lambda (nwritten)
+		((structure-ref interrupts 
+				enable-interrupts!)) 
+		nwritten))
+	  (else (wait-for-channel 
+		 (fdport-data:channel 
+		  (fdport-data (socket:inport socket)))) 
+		(loop)))))
 
-(define-foreign send-substring/errno
-  (send_substring (integer sockfd)
-		  (integer flags)
-		  (string-desc buf)
-		  (integer start)
-		  (integer end)
-		  (integer family)
-		  (string-desc name))
-  (multi-rep (to-scheme integer errno_or_false)
-	     integer))
+(import-os-error-syscall 
+  %send-substring (sockfd flags buf start end family name)
+  "send_substring")
 
 ;;;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 ;;; getsockopt syscall 
@@ -579,64 +555,31 @@
 	 (error "socket-option: integer expected ~s ~s" level option))
 	((boolean-option? option)
 	 (let ((result (%getsockopt (socket->fdes sock) level option)))
-	   (cond ((= result -1) 
-		  (error "socket-option ~s ~s ~s" sock level option))
-		 (else (not (= result 0))))))
+	   (not (= result 0))))
 	((value-option? option)
-	 (let ((result (%getsockopt (socket->fdes sock) level option)))
-	   (cond ((= result -1) 
-		  (error "socket-option ~s ~s ~s" sock level option))
-		 (else result))))
+	 (%getsockopt (socket->fdes sock) level option))
 	((linger-option? option)
-	 (receive (result/on-off time)
-		  (%getsockopt-linger (socket->fdes sock) level option)
-	    (cond ((= result/on-off -1) 
-		   (error "socket-option ~s ~s ~s" sock level option))
-		  (else (if (= result/on-off 0) #f time)))))
+	 (apply (%getsockopt-linger (socket->fdes sock) level option)
+		(lambda  (result/on-off time)
+		  (if (= result/on-off 0) #f time))))
 	((timeout-option? option)
-	 (receive (result/secs usecs)
-		  (%getsockopt-linger (socket->fdes sock) level option)
-	   (cond ((= result/secs -1) 
-		  (error "socket-option ~s ~s ~s" sock level option))
-		 (else (+ result/secs (/ usecs 1000))))))
+	 (apply (%getsockopt-timeout (socket->fdes sock) level option)
+		(lambda (result/secs usecs)
+		  (cond ((= result/secs -1) 
+			 (error "socket-option ~s ~s ~s" sock level option))
+			(else (+ result/secs (/ usecs 1000)))))))
 	(else
 	 "socket-option: unknown option type ~s" option)))
 
-(define-foreign %getsockopt/errno
-  (scheme_getsockopt (integer sockfd)
-		     (integer level)
-		     (integer optname))
-  (multi-rep (to-scheme integer errno_or_false)
-	     integer))
+(import-os-error-syscall %getsockopt (sock level option) "scheme_getsockopt")
 
-(define-errno-syscall (%getsockopt sock level option) %getsockopt/errno 
-  value)
+;;; returns (list on-off linger)
+(import-os-error-syscall %getsockopt-linger (sockfd level optname) 
+  "scheme_getsockopt_linger") 
 
-(define-foreign %getsockopt-linger/errno
-  (scheme_getsockopt_linger (integer sockfd)
-			    (integer level)
-			    (integer optname))
-  (multi-rep (to-scheme integer errno_or_false)
-	     integer) ; error/on-off
-  integer) ; linger time
-
-(define-errno-syscall 
-  (%getsockopt-linger sock level option) %getsockopt-linger/errno 
-  on-off
-  linger)
-
-(define-foreign %getsockopt-timeout/errno
-  (scheme_getsockopt_timeout (integer sockfd)
-			     (integer level)
-			     (integer optname))
-  (multi-rep (to-scheme integer errno_or_false)
-	     integer) ; error/secs
-  integer) ; usecs
-
-(define-errno-syscall 
-  (%getsockopt-timeout sock level option) %getsockopt-timeout/errno
-  secs
-  usecs)
+;;; returns (list secs usecs)
+(import-os-error-syscall %getsockopt-timeout (sockfd level optname) 
+  "scheme_getsockopt_timeout")
 
 ;;;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 ;;; setsockopt syscall 
@@ -664,38 +607,14 @@
 	(else 
 	 "set-socket-option: unknown option type")))
 
-(define-foreign %setsockopt/errno
-  (scheme_setsockopt (integer sockfd)
-		     (integer level)
-		     (integer optname)
-		     (integer optval))
-  (to-scheme integer errno_or_false))
+(import-os-error-syscall %setsockopt (sockfd level optname optval)
+  "scheme_setsockopt")
 
-(define-errno-syscall 
-  (%setsockopt sock level option value) %setsockopt/errno)
+(import-os-error-syscall %setsockopt-linger
+  (sockfd level optname on-off time) "scheme_setsockopt_linger")
 
-
-(define-foreign %setsockopt-linger/errno
-  (scheme_setsockopt_linger (integer sockfd)
-			    (integer level)
-			    (integer optname)
-			    (integer on-off)
-			    (integer time))
-  (to-scheme integer errno_or_false))
-
-(define-errno-syscall 
-  (%setsockopt-linger sock level option on-off time) %setsockopt-linger/errno)
-
-(define-foreign %setsockopt-timeout/errno
-  (scheme_setsockopt_timeout (integer sockfd)
-			     (integer level)
-			     (integer optname)
-			     (integer secs)
-			     (integer usecs))
-  (to-scheme integer errno_or_false))
-
-(define-errno-syscall 
-  (%setsockopt-timeout sock level option secs usecs) %setsockopt-timeout/errno)
+(import-os-error-syscall %setsockopt-timeout
+  (sockfd level optname secs usecs) "scheme_setsockopt_timeout")
 
 ;;;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 ;;; socket-option routines
@@ -724,6 +643,8 @@
   ((disclose hi)			; Make host-info records print like
    (list "host" (host-info:name hi))))	; #{host clark.lcs.mit.edu}.
 
+(define-exported-binding "host-info-type" type/host-info)
+
 (define (host-info arg)
   (cond ((string? arg) (name->host-info arg))
 	((socket-address? arg) (address->host-info arg))
@@ -733,44 +654,25 @@
   (if (or (not (socket-address? name)) 
 	  (not (= (socket-address:family name) address-family/internet)))
       (error "address->host-info: internet address expected ~s" name)
-      (receive (herrno name aliases addresses)
-		  (%host-address->host-info/h-errno 
-		   (socket-address:address name))
-	 (if herrno
-	     (error "address->host-info: non-zero herrno ~s ~s" name herrno)
-	     (make-host-info name 
-			     (vector->list
-			      (C-string-vec->Scheme aliases   #f))
-			     (vector->list
-			      (C-long-vec->Scheme addresses #f)))))))
+      (let ((res (%host-address->host-info/h-errno 
+		  (socket-address:address name))))
+	(if (number? res)
+	    (error "address->host-info: non-zero herrno" res name)
+	    res))))
+
+(import-lambda-definition %host-address->host-info/h-errno (name)
+  "scheme_host_address2host_info")
 
-(define-foreign %host-address->host-info/h-errno
-  (scheme_host_address2host_info (string-desc name))
-  (to-scheme integer "False_on_zero")
-  static-string	; host name
-  (C char**)    ; alias list
-  (C char**))   ; address list
-  
 (define (name->host-info name)
   (if (not (string? name))
       (error "name->host-info: string expected ~s" name)
-      (receive (herrno name aliases addresses)
-	       (%host-name->host-info/h-errno name)
-	 (if herrno
-	     (error "name->host-info: non-zero herrno ~s ~s" herrno name)
-	     (make-host-info name
-			     (vector->list
-			      (C-string-vec->Scheme aliases   #f))
-			     (vector->list
-			      (C-long-vec->Scheme   addresses #f)))))))
+      (let ((res (%host-name->host-info/h-errno name)))
+	 (if (number? res)
+	     (error "name->host-info: non-zero herrno" res name)
+	     res))))
 
-(define-foreign %host-name->host-info/h-errno
-  (scheme_host_name2host_info (string name))
-  (to-scheme integer "False_on_zero")
-  static-string	; host name
-  (C char**)    ; alias list
-  (C char**))   ; address list
-  
+(import-lambda-definition %host-name->host-info/h-errno (name) 
+  "scheme_host_name2host_info")
 
 ;;;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 ;;; network lookup
@@ -780,48 +682,30 @@
   aliases				; Alternative names
   net)					; Network number
 
+(define-exported-binding "network-info-type" type/network-info)
+
 (define (network-info arg)
   (cond ((string? arg) (name->network-info arg))
-	((socket-address? arg) (address->network-info arg))
+	((socket-address? arg) (car (socket-address:address arg)))
 	(else 
 	 (error "network-info: string or socket-address expected ~s" arg))))
 
-(define (address->network-info name)
-  (if (not (integer? name))
-      (error "address->network-info: integer expected ~s" name)
-      (let ((name (integer->string name))
-	    (net (make-string 4)))
-	(receive (result name aliases)
-		 (%net-address->network-info name net)
-	  (make-network-info name 
-			     (vector->list
-			      (C-string-vec->Scheme aliases #f))
-			     (string->integer net))))))
-		  
-(define-foreign %net-address->network-info
-  (scheme_net_address2net_info (string-desc name) (string-desc net))
-  (to-scheme integer "False_on_zero")
-  static-string	; net name
-  (C char**))   ; alias list
+(define (address->network-info addr)
+  (if (not (integer? addr))
+      (error "address->network-info: integer expected ~s" addr)
+      (%net-address->network-info addr)))
 
-  
+(import-lambda-definition %net-address->network-info (addr) 
+  "scheme_net_address2net_info")
+ 
 (define (name->network-info name)
   (if (not (string? name))
       (error "name->network-info: string expected ~s" name)
-      (let ((net (make-string 4)))
-	(receive (result name aliases)
-		 (%net-name->network-info name net)
-	   (make-network-info name
-			      (vector->list
-			       (C-string-vec->Scheme aliases #f))
-			      (string->integer net))))))
-		  
-(define-foreign %net-name->network-info
-  (scheme_net_name2net_info (string name) (string-desc net))
-  (to-scheme integer "False_on_zero")
-  static-string	 ; net name
-  (C char**))    ; alias list
+      (%net-name->network-info name)))
 
+(import-lambda-definition %net-name->network-info (name)
+  "scheme_net_name2net_info")
+		  
 ;;;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 ;;; service lookup
 ;;;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
@@ -831,6 +715,8 @@
   port					; Port number
   protocol)				; Protocol name
 
+(define-exported-binding "service-info-type" type/service-info)
+
 (define (service-info . args)
   (apply (cond ((string?  (car args)) name->service-info)
 	       ((integer? (car args)) port->service-info)
@@ -844,35 +730,26 @@
 	  ((not (string? proto))
 	   (error "port->service-info: string expected ~s" proto))
 	  (else
-	   (receive (result name aliases port protocol)
-		    (%service-port->service-info name proto)
-	     (make-service-info name 
-				(vector->list (C-string-vec->Scheme aliases #f))
-				port
-				protocol))))))
-		  
-(define-foreign %service-port->service-info
-  (scheme_serv_port2serv_info (integer name) (string  proto))
-  (to-scheme integer "False_on_zero")
-  static-string	 ; service name
-  (C char**)     ; alias list
-  integer        ; port number
-  static-string) ; protocol name
-  
+	   (%service-port->service-info name (if (equal? "" proto) 
+						 #f 
+						 proto))))))
+	     
+(import-lambda-definition %service-port->service-info (port proto)
+  "scheme_serv_port2serv_info")
   
 (define (name->service-info name . maybe-proto)
-  (receive (result name aliases port protocol)
-      (%service-name->service-info name (:optional maybe-proto ""))
-    (make-service-info name (vector->list (C-string-vec->Scheme aliases #f))
-		       port protocol)))
-		  
-(define-foreign %service-name->service-info
-  (scheme_serv_name2serv_info (string name) (string proto))
-  (to-scheme integer "False_on_zero")
-  static-string	 ; service name
-  (C char**)     ; alias list
-  integer        ; port number
-  static-string) ; protocol name
+  (let ((proto (:optional maybe-proto "")))
+    (cond ((not (string? name))
+	   (error "name->service-info: integer expected ~s" name))
+	  ((not (string? proto))
+	   (error "name->service-info: string expected ~s" proto))
+	  (else
+	   (%service-name->service-info name (if (equal? "" proto) 
+						 #f 
+						 proto))))))
+
+(import-lambda-definition %service-name->service-info (name proto)
+  "scheme_serv_name2serv_info")
 
 ;;;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
 ;;; protocol lookup
@@ -882,6 +759,8 @@
   aliases				; Alternative names
   number)				; Protocol number
 
+(define-exported-binding "protocol-info-type" type/protocol-info)
+
 (define (protocol-info arg)
   (cond ((string? arg)  (name->protocol-info arg))
 	((integer? arg) (number->protocol-info arg))
@@ -890,83 +769,16 @@
 (define (number->protocol-info name) 
   (if (not (integer? name))
       (error "number->protocol-info: integer expected ~s" name)
-      (receive (result name aliases protocol)
-	       (%protocol-port->protocol-info name)
-	 (make-protocol-info name 
-			     (vector->list
-			      (C-string-vec->Scheme aliases #f))
-			     protocol))))
+      (%protocol-port->protocol-info name)))
 
-(define-foreign %protocol-port->protocol-info
-  (scheme_proto_num2proto_info (integer name))
-  (to-scheme integer "False_on_zero")
-  static-string	; protocol name
-  (C char**)    ; alias list
-  integer)      ; protocol number
+(import-lambda-definition %protocol-port->protocol-info (name)
+  "scheme_proto_num2proto_info")
   
 (define (name->protocol-info name)
   (if (not (string? name))
       (error "name->protocol-info: string expected ~s" name)
-      (receive (result name aliases protocol)
-	       (%protocol-name->protocol-info name)
-	 (make-protocol-info name
-			     (vector->list
-			      (C-string-vec->Scheme aliases #f))
-			     protocol))))
-		  
-(define-foreign %protocol-name->protocol-info
-  (scheme_proto_name2proto_info (string name))
-  (to-scheme integer "False_on_zero")
-  static-string ; protocol name
-  (C char**)    ; alias list
-  integer)      ; protocol number
+      (%protocol-name->protocol-info name)))
 
-;;;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
-;;; Lowlevel junk 
-;;;-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
-;; Used to pull address list back
-;; based on C-string-vec->Scheme from cig/libcig.scm
-(define (C-long-vec->Scheme cvec veclen) ; No free.
-  (let ((vec (make-vector (or veclen (%c-veclen-or-false cvec) 0))))
-    (mapv! (lambda (ignore) (make-string 4)) vec)
-    (%set-long-vector-carriers! vec cvec)
-    (mapv! string->integer vec)))
+(import-lambda-definition %protocol-name->protocol-info (name)
+  "scheme_proto_name2proto_info")
 
-(define (integer->string num32)
-  (let* ((str   (make-string 4))
-	 (num24 (arithmetic-shift num32 -8))
-	 (num16 (arithmetic-shift num24 -8))
-	 (num08 (arithmetic-shift num16 -8))
-	 (byte0 (bitwise-and #b11111111 num08))
-	 (byte1 (bitwise-and #b11111111 num16))
-	 (byte2 (bitwise-and #b11111111 num24))
-	 (byte3 (bitwise-and #b11111111 num32)))
-    (string-set! str 0 (ascii->char byte0))
-    (string-set! str 1 (ascii->char byte1))
-    (string-set! str 2 (ascii->char byte2))
-    (string-set! str 3 (ascii->char byte3))
-    str))
-
-(define (string->integer str)
-  (+ (arithmetic-shift(char->ascii(string-ref str 0))24)
-     (arithmetic-shift(char->ascii(string-ref str 1))16)
-     (arithmetic-shift(char->ascii(string-ref str 2)) 8)
-     (char->ascii(string-ref str 3))))
-
-;; also from cig/libcig.scm
-(define-foreign %c-veclen-or-false
-  (veclen ((C "const long * ~a") c-vec)); redefining can we open cig-aux?
-  desc) ; integer or #f if arg is NULL.
-
-;; also from cig/libcig.scm
-(define-foreign %set-long-vector-carriers!
-  (set_longvec_carriers (vector-desc svec)
-			((C "long const * const * ~a") cvec))
-  ignore)
-
-;; also from cig/libcig.scm
-(define (mapv! f v)
-  (let ((len (vector-length v)))
-    (do ((i 0 (+ i 1)))
-	((= i len) v)
-      (vector-set! v i (f (vector-ref v i))))))
