@@ -1,13 +1,13 @@
 ;;; Delimited readers
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;; These procedures run their inner I/O loop in a C primitive, so they
-;;; should be quite fast.
+;;; These procedures ran their inner I/O loop in a C primitive in
+;;; earlier versions of scsh. In a multi-threaded environment this
+;;; causes lots of trouble in case the operation would
+;;; block. Therefore the current implementation runs in Scheme but
+;;; operates directly on the buffer of the port for speed. This also
+;;; allows us to implement the push-back behaviour without a peek/read
+;;; pair.
 ;;;
-;;; N.B.:
-;;; The C primitives %READ-DELIMITED-FDPORT!/ERRNO and 
-;;; %SKIP-CHAR-SET-FDPORT/ERRNO rely on knowing the representation of
-;;; character sets. If these are changed from their current representation, 
-;;; this code must be changed as well. 
 
 ;;; (read-delimited delims [port delim-action])
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
@@ -30,12 +30,11 @@
 (define (read-delimited delims . args)
   (let-optionals args ((port         (current-input-port))
 		       (delim-action 'trim))
-    (let ((substr (lambda (s end)		; Smart substring.
-		    (if (= end (string-length s)) s
-			(substring s 0 end))))
-	  (delims (->char-set delims))
-	  (gobble? (not (eq? delim-action 'peek))))
-	
+	(let ((substr (lambda (s end)		; Smart substring.
+			(if (= end (string-length s)) s
+			    (substring s 0 end))))
+	      (delims (->char-set delims))
+	      (gobble? (not (eq? delim-action 'peek))))
       ;; BUFLEN is total amount of buffer space allocated to date.
       (let lp ((strs '()) (buflen 80) (buf (make-string 80)))
 	(receive (terminator num-read)
@@ -154,101 +153,93 @@
 ;;; the input stream. Otherwise, it is left in place for a following input
 ;;; operation.
 
+
+(define (port-buffer-read-delimited delims buf gobble? port start end)
+  (obtain-port-lock port)
+  (let ((the-port-limit (port-limit port)))
+    (let lp ((i start) (lp-port-index (port-index port))) 
+      (cond ((port-pending-eof? port)
+	     (set-port-index! port lp-port-index)
+	     (release-port-lock port)
+	     (values (eof-object) (- i start)))
+	    ((>= i end)
+	     (set-port-index! port lp-port-index)
+	     (release-port-lock port)
+	     (values #f (- i start)))
+	    ((< lp-port-index the-port-limit)
+	     (let ((the-read-char 
+		    (ascii->char (byte-vector-ref 
+				  (port-buffer port) lp-port-index))))
+	       (if (char-set-contains? delims the-read-char)
+		   (begin 
+		     (if gobble? 
+			 (set-port-index! port (+ lp-port-index 1))
+			 (set-port-index! port lp-port-index))
+		     (release-port-lock port)
+		     (values the-read-char (- i start)))
+		   (begin
+		     (string-set! buf i the-read-char)
+		     (lp (+ i 1) (+ lp-port-index 1))))))
+	    (else  (set-port-index! port 0)
+		   (set-port-limit! port 0)
+		   (release-port-lock port)
+		   (values 'port-buffer-exhausted (- i start)))))))
+		 
+
+
 (define (%read-delimited! delims buf gobble? . args)
   (let-optionals args ((port  (current-input-port))
 		       (start 0)
 		       (end   (string-length buf)))
 
-    (check-arg input-port? port %read-delimited!)	; Arg checking.
-    (check-arg char-set? delims %read-delimited!)	; Required, since
-    (if (bogus-substring-spec? buf start end)		; we're calling C.
-	(error "Illegal START/END substring indices"
-	       buf start end %read-delimited!))
-
     (let* ((delims (->char-set delims))
 	   (sdelims (char-set:s delims)))
-
-      (if (fdport? port)
-
-	  ;; Direct C support for Unix file ports -- zippy quick.
-	  (let lp ((start start) (total 0))
-	    (receive (terminator num-read)
-		     (%read-delimited-fdport!/errno sdelims buf gobble?
-						    port start end)
-	      (let ((total (+ num-read total)))
-		(cond ((not (integer? terminator)) (values terminator total))
-		      ((= terminator errno/intr) (lp (+ start num-read) total))
-		      (else (errno-error terminator %read-delimited!
-					 num-read total
-					 delims buf gobble? port start end))))))
-
-	  ;; This is the code for other kinds of ports.
-	  ;; Mighty slow -- we read each char twice (peek first, then read).
-	  (let lp ((i start))
-	    (let ((c (peek-char port)))
-	      (cond ((or (eof-object? c)	; Found terminating char or eof
-			 (char-set-contains? delims c))
-		     (if gobble? (read-char port))
-		     (values c (- i start)))
-
-		    ((>= i end)			; Filled the buffer.
-		     (values #f (- i start)))
-			
-		    (else (string-set! buf i (read-char port))
-			  (lp (+ i 1))))))))))
+      (let lp ((start start) (total 0))
+      (receive (terminator num-read) 
+	       (port-buffer-read-delimited delims buf gobble? port start end)
+	       (if (not (eq? terminator 'port-buffer-exhausted))
+		   (values terminator (+ num-read total))
+		   (begin (peek-char port)  ; kludge to fill the buffer
+			  (lp (+ start num-read) (+ total num-read)))))))))
+			  
 
 
-(foreign-source
-  "#include <sys/types.h>"
-  ""
-  "/* Make sure foreign-function stubs interface to the C funs correctly: */"
-  "#include \"fdports1.h\""
-  "" "")
-
-(define-foreign %read-delimited-fdport!/errno (read_delim (string delims)
-							  (var-string buf)
-							  (bool gobble?)
-							  (desc port)
-							  (fixnum start)
-							  (fixnum end))
-  desc	; int => errno; char => terminating char; eof-object; #f => buf ovflow
-  fixnum)   ; number of chars read into BUF.
-
-
-(define-foreign %skip-char-set-fdport/errno (skip_chars (string skip-set)
-							(desc port))
-  desc		; int => errno; #f => win.
-  fixnum)	; number of chars skipped.
+; overwrites port-index :-(
+(define (push-back port char)
+  (byte-vector-set! (port-buffer port) (port-index port) (char->ascii char))
+  (set-port-limit! port (+ (port-limit port) 1)))
 
 
 (define (skip-char-set skip-chars . maybe-port)
   (let* ((port (:optional maybe-port (current-input-port)))
-	 (cset (->char-set skip-chars))
-	 (scset (char-set:s cset)))
-
-      (cond ((not (input-port? port))
-	     (error "Illegal value -- not an input port." port))
-
-	     ;; Direct C support for Unix file ports -- zippy quick.
-	     ((fdport? port)
-	      (let lp ((total 0))
-		(receive (err num-read) (%skip-char-set-fdport/errno scset port)
-		  (let ((total (+ total num-read)))
-		    (cond ((not err) total)
-			  ((= errno/intr err) (lp total))
-			  (errno-error err skip-char-set cset port total))))))
-
-	     ;; This is the code for other kinds of ports.
-	     ;; Mighty slow -- we read each char twice (peek first, then read).
-	     (else (let lp ((i 0))
-		     (let ((c (peek-char port)))
-		       (cond ((and (char? c) (char-set-contains? cset c))
-			      (read-char port)
-			      (lp (+ i 1)))
-			     (else i))))))))
-
-
+	(cset (->char-set skip-chars))
+	(scset (char-set:s cset)))
+    
+    (let lp ((total 0))
+      (receive (succ num-read) (buffer-skip-char-set cset port)
+	       (if (not succ)
+		   (+ total num-read) ; eof
+		   (begin (peek-char port); kludge to fill the buffer
+			  (lp  (+ total num-read))))))))
 
+(define (buffer-skip-char-set cset port)
+  (let ((the-port-limit (port-limit port)))
+    (let lp ((lp-port-index (port-index port)) (i 0))
+      (cond ((port-pending-eof? port)
+	     (set-port-index! port lp-port-index)
+	     (values #f i))
+	    ((< lp-port-index the-port-limit)
+	     (let ((the-read-char 
+		    (ascii->char (byte-vector-ref 
+				  (port-buffer port) lp-port-index))))
+	       (cond ((char-set-contains? cset the-read-char)
+		      (lp (+ lp-port-index 1) (+ i 1)))
+		     (else 
+		      (set-port-index! port lp-port-index)
+		      (values #f i)))))
+	    (else (set-port-index! port 0)
+		  (set-port-limit! port 0)
+		  (values 'port-buffer-exhausted i))))))
 
 ;;; (read-line [port delim-action])
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
