@@ -498,7 +498,7 @@
 			 (values re #f))))
 
 		  ((re-submatch? re)
-		   (receive (new-body body-changed?) (recur (re-submatch? re))
+		   (receive (new-body body-changed?) (recur (re-submatch:body re))
 		     (if body-changed?
 			 (values (%make-re-submatch new-body
 						    (re-submatch:pre-dsm re)
