@@ -1,7 +1,6 @@
 ;;; The module interfaces for scsh.
 ;;; Copyright (c) 1994 by Olin Shivers and David Albertz.
 ;;; Copyright (c) 1994 by Brian D. Carlstrom
-;;; See file COPYING.
 
 (define-interface posix-fdflags-interface
   (export ((open/read
@@ -113,6 +112,7 @@
   (compound-interface buffered-io-flags-interface
 		      (export close
 			      close-after
+			      current-error-port
 			      error-output-port
 			      dup
 			      dup->inport
@@ -125,6 +125,9 @@
 			      fdes-status
 			      set-fdes-status
 
+			      init-fdports!  ;added by JMG
+			      port->channel  ;overwrites channel-i/o
+
 			      force-output
 			      set-port-buffering
 			      bufpol/block
@@ -137,8 +140,8 @@
 			      seek/delta
 			      seek/end
 
-			      select
-			      select!
+;			      select
+;			      select!
 
 			      flush-all-ports
 			      y-or-n?
@@ -186,11 +189,11 @@
 			      (with-current-input-port :syntax)
 			      with-current-output-port*
 			      (with-current-output-port :syntax)
-			      with-error-output-port*
-			      (with-error-output-port :syntax)
+			      with-current-error-port*
+			      (with-current-error-port :syntax)
 			      set-current-input-port!
 			      set-current-output-port!
-			      set-error-output-port!
+			      set-current-error-port!
 
 			      stdports->stdio
 			      stdio->stdports
@@ -313,6 +316,7 @@
 	  pid->proc
 
 	  autoreap-policy
+	  with-autoreaping
 	  reap-zombies
 	  
 	  wait
@@ -325,8 +329,8 @@
 	  wait/poll
 	  wait/stopped-children
 
-	  sleep
-	  sleep-until
+	  process-sleep
+	  process-sleep-until
 
 	  call-terminally
 	  halts?))
@@ -337,11 +341,17 @@
 	  set-umask
 	  with-umask*
 	  (with-umask :syntax)
+	  with-umask-aligned*
+	  (with-umask-aligned :syntax)
 
+	  process-chdir
+	  process-cwd
 	  chdir
 	  cwd
 	  with-cwd*
 	  (with-cwd :syntax)
+	  with-cwd-aligned*
+	  (with-cwd-aligned :syntax)
 
 	  pid
 	  parent-pid
@@ -357,6 +367,7 @@
 	  user-supplementary-gids
 	  set-uid
 	  set-gid
+	  set-effective-gid
 
 	  system-name
 	  process-times
@@ -394,12 +405,15 @@
 (define-interface scsh-signals-interface
   (export signal-process
 	  signal-process-group
-	  pause-until-interrupt
-	  itimer))
+	  ;; JMG: this syscalls doesn't cooperate with the thread-system 
+	  ;;	  pause-until-interrupt	  
+	  itimer  ;; now defined in low-interrupt as a artificial interrupt
+))
 
 
 (define-interface scsh-environment-interface
-  (export setenv
+  (export install-env
+	  setenv
 	  getenv
 	  env->alist
 	  alist->env
@@ -579,7 +593,13 @@
 	  check-arg conjoin disjoin negate compose reverse! call/cc
 	  deprecated-proc
 	  deposit-bit-field
-	  real->exact-integer))
+	  real->exact-integer
+	  make-reinitializer
+	  run-as-long-as))
+
+(define-interface weak-tables-interface
+  (export make-weak-table weak-table-set! weak-table-ref weak-table-walk
+	  strengthen-weak-table-ref weaken-weak-table-ref))
 
 ;;; semi-standard network magic numbers
 ;;; should be available on all platforms
@@ -637,6 +657,7 @@
 	  unix-address->socket-address
 	  socket-address->unix-address
 	  create-socket
+	  port->socket
 	  close-socket
 	  bind-socket
 	  connect-socket
@@ -654,7 +675,7 @@
 	  send-message/partial
 	  socket-option
 	  set-socket-option
-
+	  
 	  host-info
 	  host-info?
 	  host-info:name
@@ -685,7 +706,7 @@
 
 
 (define-interface scsh-field-reader-interface
-  (export join-strings	; Obsolete. Has become SRFI-13's STRING-JOIN.
+  (export join-strings
 	  field-splitter infix-splitter suffix-splitter sloppy-suffix-splitter
 	  record-reader
 	  field-reader
@@ -983,6 +1004,9 @@
   (export signal->interrupt
 	  interrupt-set
 
+	  interrupt-handlers-vector  ; JMG: replaces vm vector
+	  with-scsh-sighandlers
+
 	  (with-enabled-interrupts :syntax)
 	  with-enabled-interrupts*
 	  enabled-interrupts
@@ -991,12 +1015,12 @@
 	  set-interrupt-handler
 	  interrupt-handler
 
-	  %set-unix-signal-handler
-	  %unix-signal-handler
-
+	  ;; reexported from low-interrupt
 	  interrupt/alrm interrupt/alarm
 	  interrupt/int  interrupt/keyboard
-	  interrupt/memory-shortage
+;	  interrupt/memory-shortage
+	  interrupt/post-gc
+	  interrupt/i/o-completion
 	  interrupt/chld
 	  interrupt/cont
 	  interrupt/hup	
@@ -1016,24 +1040,83 @@
 	  interrupt/xcpu	
 	  interrupt/xfsz))
 
-(define-interface odbc-interface
-  (export current-db
-	  call/db
-	  with-current-db*
-	  with-current-db
-	  with-open-db*
-	  with-open-db
-	  string->sql-command
-	  open-db
-	  set-current-db!
-	  close-db
- 	  execute-sql
-	  fetch-row
-	  fetch-rows
-	  fetch-all
-	  close-cursor
-	  commit
-	  rollback
-	  cancel
-	  cursor-name?
-	  set-cursor-name!))
+(define-interface sigevents-interface
+  (export most-recent-sigevent
+	  sigevent?
+	  next-sigevent
+	  next-sigevent-set
+	  next-sigevent/no-wait
+	  next-sigevent-set/no-wait
+	  sigevent-type
+	  schedule-timer-interrupt!))
+
+
+(define-interface low-interrupt-interface
+  (export number-of-interrupts
+	  interrupt/alrm interrupt/alarm
+	  interrupt/int  interrupt/keyboard
+	  interrupt/post-gc
+	  interrupt/i/o-completion
+	  interrupt/chld
+	  interrupt/cont
+	  interrupt/hup	
+	  interrupt/quit	
+	  interrupt/term	
+	  interrupt/tstp	
+	  interrupt/usr1	
+	  interrupt/usr2	
+	  interrupt/info
+	  interrupt/io	
+	  interrupt/poll	
+	  interrupt/prof	
+	  interrupt/pwr	
+	  interrupt/urg	
+	  interrupt/vtalrm
+	  interrupt/winch
+	  interrupt/xcpu	
+	  interrupt/xfsz
+	  interrupt-set
+	  interrupt-in-set?
+	  insert-interrupt
+	  remove-interrupt
+	  full-interrupt-set))
+
+(define-interface dot-locking-interface
+  (export obtain-dot-lock release-dot-lock
+	  break-dot-lock
+	  (with-dot-lock :syntax)
+	  with-dot-lock*))
+
+(define-interface syslog-interface
+  (export (syslog-option :syntax)
+	  syslog-option?
+
+	  make-syslog-options
+	  syslog-options?
+	  (syslog-options :syntax)
+
+	  (syslog-facility :syntax)
+	  syslog-facility?
+
+	  (syslog-level :syntax)
+	  syslog-level?
+
+	  make-syslog-mask
+	  syslog-mask?
+	  (syslog-mask :syntax)
+	  syslog-mask-all
+	  syslog-mask-upto
+
+	  with-syslog-destination
+	  set-syslog-destination!
+
+	  syslog))
+
+(define-interface syslog-channels-interface
+  (export open-syslog-channel
+	  close-syslog-channel
+	  set-syslog-channel!
+	  with-syslog-channel))
+
+(define-interface crypt-interface
+  (export crypt))
\ No newline at end of file
