@@ -1,5 +1,5 @@
 ;;; The packages that scsh uses/defines.
-;;; Copyright (c) 1994 by Olin Shivers. See file COPYING.
+;;; Copyright (c) 1994 by Olin Shivers.
 
 ;;; Note: field-reader package (fr.scm) and here docs use READ-LINE.
 ;;; It is defined in rdelim.scm.
@@ -28,28 +28,35 @@
 
 (define-structure error-package (export error warn)
   (open signals)
-  (optimize auto-integrate)
+;  (optimize auto-integrate)
   )
 
 
 (define-structure scsh-utilities scsh-utilities-interface
-  (open bitwise error-package loopholes let-opt scheme)
+  (open bitwise error-package loopholes let-opt scheme define-record-types 
+	records
+	threads threads-internal placeholders)
   (files utilities)
-  (optimize auto-integrate)
+;  (optimize auto-integrate)
   )
 
+(define-structure weak-tables weak-tables-interface
+  (open scheme
+	weak
+	tables)
+  (files weaktables))
 
 ;;; This guy goes into the FOR-SYNTAX part of scsh's syntax exports.
 (define-structure scsh-syntax-helpers
   (export transcribe-extended-process-form)
   (open receiving	; receive
 	error-package
-	syntactic	; generated?
+	names	        ; generated? by JMG
 	scsh-utilities	; check-arg
 	scheme
 	)
   (files syntax-helpers)
-  (optimize auto-integrate)
+;  (optimize auto-integrate)
   )
 
 
@@ -57,15 +64,16 @@
 (define-structure buffered-io-flags buffered-io-flags-interface
   (open defenum-package scheme)
   (files (machine bufpol))
-  (optimize auto-integrate)
+;  (optimize auto-integrate)
   )
 
 
+
 (define-structures ((tty-flags tty-flags-interface)
 		    (scsh-internal-tty-flags scsh-internal-tty-flags-interface))
   (open scheme ascii bitwise)
   (files (machine tty-consts))
-  (optimize auto-integrate)
+;  (optimize auto-integrate)
   )
 
 
@@ -76,14 +84,13 @@
 (define-structure partial-s48-ports
   (export current-input-port current-output-port
 	  newline
-	  error-output-port
+	  current-error-port
 
 	  $current-input-port
 	  $current-output-port
-	  $error-output-port)
-  (open ports)
-  (optimize auto-integrate)
-  )
+	  $current-error-port
+	  $current-noise-port)
+  (open ports i/o))
 
 ;;; The scsh-level-0 package is for implementation convenience.
 ;;; The scsh startup and top-level modules need access to scsh
@@ -92,13 +99,13 @@
 ;;; imported by these two modules. These modules all collectively
 ;;; export the whole scsh enchilada.
 
-(define-structures
+(define-structures 
   ((scsh-level-0
     (compound-interface posix-fdflags-interface
 			posix-errno-interface
 			posix-signals-interface
-			sockets-network-interface   ; Standard Network Interface
-			os-extras-interface         ; Extra stuff from OS.
+			sockets-network-interface ; Standard Network Interface
+			os-extras-interface ; Extra stuff from OS.
 			scsh-delimited-readers-interface
 			scsh-errors-interface
 			scsh-io-interface
@@ -114,37 +121,42 @@
 			scsh-file-names-interface
 			scsh-misc-interface
 			scsh-high-level-process-interface
-			scsh-time-interface         ; New in release 0.2
-			scsh-sockets-interface      ; New in release 0.3
-			tty-interface               ; New in release 0.4
+			scsh-time-interface ; new in 0.2
+			scsh-sockets-interface ; new in 0.3
+			tty-interface ; new in 0.4
 			scsh-version-interface
 			char-set-interface
 			signal-handler-interface
 			;; This stuff would probably be better off kept
 			;; in separate modules, but we'll toss it in for now.
-			(interface-of ascii)        ; char<->ascii
+			(interface-of ascii) ; char<->ascii
 			string-ports-interface
+			syslog-interface
+			crypt-interface
 			))
    (scsh-level-0-internals (export set-command-line-args!
 				   init-scsh-hindbrain
+				   initialize-cwd
 				   init-scsh-vars))
-; (scsh-regexp-package scsh-regexp-interface)
-   )
+;   (scsh-regexp-package scsh-regexp-interface)
+)
   (for-syntax (open scsh-syntax-helpers scheme))
-  (open externals
+  (access rts-sigevents sigevents threads)
+  (open enumerated
+	defenum-package
+	external-calls           ;JMG new FFI
 	structure-refs
-	cig-aux
 	receiving
 	defrec-package
-	define-foreign-syntax
+	define-record-types
 	formats
-	os-dependent                        ; OS dependent cpde
-	buffered-io-flags                   ; stdio dependent
+	os-dependent		; OS dependent stuff
+	buffered-io-flags	; stdio dependent
 	ascii
 	records
 	extended-ports
 	partial-s48-ports
-;	ports
+	ports
 	build
 	bigbit
 	bitwise
@@ -152,19 +164,22 @@
 	conditions
 	scsh-utilities
 	handle
-	fluids
-	weak
+	fluids thread-fluids
+	weak-tables
 
-	scsh-char-set-low-level-lib	    ; rdelim.scm needs it.
+	scsh-char-set-low-level-lib         ; rdelim.scm needs it.
 ;	scsh-regexp-package
 ;	scsh-regexp-internals
 	char-set-lib
 	scsh-version
 	tty-flags
-	scsh-internal-tty-flags             ; Not exported
-	let-opt                             ; optional-arg parsing & defaulting
+	scsh-internal-tty-flags	; Not exported
+
+	syslog
+
+	let-opt			; optional-arg parsing & defaulting
 	
-	interrupts                          ; signal handler code
+	architecture     ; Was this by JMG ??
 
 	re-level-0
 	rx-syntax
@@ -175,45 +190,57 @@
 	loopholes		; For my bogus CALL-TERMINALLY implementation.
 
 	scheme
-	)
-
-  (access command-processor
+	
+	low-interrupt            ; for sighandler and procobj
+	i/o                  ; all these seem to be for scsh-0.6 JMG
+	i/o-internal
+	channels channel-i/o
+	low-channels
+	byte-vectors
+	threads locks placeholders
+	primitives
+	escapes
+	command-levels
+	features
+	general-tables
+	simple-syntax)  
+  (for-syntax (open scsh-syntax-helpers scheme))
+  (access interrupts
+	  sort
+	  command-processor
 	  escapes
-	  ports                             ; S48's force-output
+	  i/o		; S48's force-output
+	  exceptions    ; signal-exception
 	  formats
-	  records                           ; I don't think this is necessary. !!!
-	  scheme)                         ; For accessing the normal I/O operators.
-
-  (begin (define set-batch-mode?!
-	   (structure-ref command-processor set-batch-mode?!))
-	 (define with-continuation (structure-ref escapes with-continuation)))
-
+	  threads-internal 
+	  records	; I don't think this is necessary. !!!
+	  scheme)	; For accessing the normal I/O operators.
   (files syntax
 	 syscalls
-	 select
+;	 select
 	 fname
 	 stringcoll
 	 scsh-condition
 	 rw
 	 newports
 	 fdports
-	 procobj                            ; New in release 0.4
-	 (machine waitcodes)                ; OS dependent code
+	 procobj		; New in release 0.4.
+	 (machine waitcodes)	; OS dependent code.
 	 filesys
 	 fileinfo
 	 glob
 ;	 filemtch
-	 time                               ; New in release 0.2
+	 time			; New in release 0.2.
 	 (machine time_dep)
-	 network                            ; New in release 0.3
-	 endian                             ; New in release 0.4
-	 flock                              ; New in release 0.4
-	 tty                                ; New in release 0.4
-	 pty                                ; New in release 0.4
-	 sighandlers                        ; New in release 0.5
+	 network		; New in release 0.3.
+	 endian			; New in release 0.4.
+	 flock			; New in release 0.4.
+	 tty			; New in release 0.4.
+	 pty			; New in release 0.4.
+	 sighandlers		; New in release 0.5.
 	 scsh
 ;	 re
-	 rdelim
+	 rdelim  
 	 )
 ;  (optimize auto-integrate)
   )
@@ -222,64 +249,84 @@
   (open records scheme)
   (for-syntax (open scheme error-package receiving))
   (files defrec)
-  (optimize auto-integrate)
+;  (optimize auto-integrate)
   )
 
 (define-structure defenum-package (export (define-enum-constant  :syntax)
-					  (define-enum-constants :syntax))
+					  (define-enum-constants :syntax)
+					  (define-enum-constants-from-zero 
+					    :syntax))
   (open scheme)
   (files enumconst)
-  (optimize auto-integrate)
+;  (optimize auto-integrate)
   )
 
 ;;; This code opens so many modules of gruesome, low-level S48 internals 
 ;;; that these two modules are segregated into separate packages, each
 ;;; exporting just two definitions.
 
-(define-structure scsh-startup-package (export dump-scsh-program dump-scsh)
+(define-structure scsh-startup-package (export dump-scsh-program 
+					       dump-scsh
+					       make-scsh-starter
+					       scsh-stand-alone-resumer)
   (open scsh-level-0-internals	; init-scsh-* set-command-line-args!
 	scsh-level-0		; error-output-port command-line-arguments
 	scsh-top-package	; parse-switches-and-execute
 	handle			; with-handler
-	command-processor	; user-context
+	command-levels  	; user-context
 	write-images		; write-image
 	build-internals		; simple-condition-handler
 	low-level		; flush-the-symbol-table!
 	command-processor	; command-output
+	package-commands-internal
 	filenames		; translate
-	scheme-level-2-internal	; usual-resumer
+	usual-resumer           ; usual-resumer
+	environments            ; with-interaction-environment
+	fluids-internal            ; JMG: get-dynamic-env 
+	scsh-utilities
+	interrupts
+	low-interrupt
+	sigevents
+	primitives
 	scheme)
-  (files startup)
-  (optimize auto-integrate)
-  )
+  (files startup))
 
-(define-structure scsh-top-package (export parse-switches-and-execute repl)
+(define-structure scsh-top-package (export parse-switches-and-execute 
+					   repl )
   (open command-processor
+	command-levels          ; with-new-session
 	ensures-loaded
 	environments
 	error-package
 	evaluation
 	extended-ports
+	fluids
 	interfaces
+	sigevents                     
+	low-interrupt
+	fluids-internal            ; JMG: get-dynamic-env 
+	handle                     ; JMG: with-handler
+;	package-commands
 	interrupts
+	i/o
 	package-commands-internal
 	package-mutation
 	packages
 	receiving
 	scsh-version
-	scsh-level-0                    ; with-current-input-port error-output-port
-                                    ; with-current-output-port exit
-	scsh-level-0-internals          ; set-command-line-args! init-scsh-vars
-	handle list-lib                 ; For lib-search facility
+	scsh-level-0		; with-current-input-port error-output-port
+				; with-current-output-port exit
+	scsh-level-0-internals	; set-command-line-args! init-scsh-vars
+	threads
+	list-lib                ; any
+	root-scheduler          ; scheme-exit-now
 	scheme)
-  (files top meta-arg)
-  (optimize auto-integrate)
-  )
+  (files top meta-arg))
 
 
 (define-structure field-reader-package scsh-field-reader-interface
-  (open receiving		; receive
-	scsh-utilities		; nth & deprecated-proc (for join-strings)
+  (open receiving             ; receive
+	scsh-utilities          ; nth
 	error-package		; error
 	string-lib		; string-join for obsolete join-strings
 	scsh-level-0		; delimited readers
@@ -292,7 +339,6 @@
   ;; Handle a little bit of backwards compatibility.
   (begin (define join-strings (deprecated-proc string-join 'join-strings
 					       "Use SRFI-13 STRING-JOIN.")))
-  (optimize auto-integrate)
   )
 
 
@@ -308,9 +354,9 @@
 	sre-syntax-tools
 	scheme
 	)
-  (files awk)
-  (optimize auto-integrate)
-  )
+  (files awk) 
+;  (optimize auto-integrate)
+)
 	 
 
 (define-structure awk-package awk-interface
@@ -328,24 +374,6 @@
   (begin (define-syntax awk 
            (syntax-rules () ((awk body ...) (awk/posix-string body ....))))))
 
-(define-structure odbc-package odbc-interface
-  (open scsh-level-0
-        defrec-package
-        define-foreign-syntax
-        let-opt
-        conditions
-        fluids
-        handle
-        signals
-        scheme)
-  (files
-	 (odbc odbc0)                       ; New in release 0.5.3
-	 (odbc odbc1)                       ; New in release 0.5.3
-	 (odbc odbc2)                       ; New in release 0.5.3
-     )
-;  (optimize auto-integrate)
-)
-
 (define-structure scsh
   (compound-interface (interface-of scsh-level-0)
 		      (interface-of scsh-startup-package)
@@ -356,9 +384,9 @@
 ;		      scsh-dbm-interface
 		      (export repl)
 		      awk-interface
-		      odbc-interface
 		      char-predicates-interface; Urk -- Some of this is R5RS!
 		      obsolete-char-set-interface
+		      dot-locking-interface
 		      )
 
   (open structure-refs
@@ -371,14 +399,14 @@
 	scsh-startup-package
 ;	dbm
 	awk-package
-	odbc-package
 	field-reader-package
 	char-predicates-lib	; Urk -- Some of this is R5RS!
+	dot-locking
 	scheme)
 
   (access scsh-top-package)
   (begin (define repl (structure-ref scsh-top-package repl)))
-  (optimize auto-integrate)
+;  (optimize auto-integrate)
   )
 
 (define-structure scsh-here-string-hax (export)
@@ -387,6 +415,65 @@
 	scsh		; Just need the delimited readers.
 	features	; make-immutable!
 	scheme)
-  (files here)
-  (optimize auto-integrate)
-  )
+  (files here))
+
+(define-structure sigevents sigevents-interface
+  (open scsh-level-0
+	scheme
+	structure-refs
+	low-interrupt
+	rts-sigevents)
+  (files event))
+
+(define-structure simple-syntax (export define-simple-syntax)
+  (open scheme)
+ (begin (define-syntax define-simple-syntax
+	  (syntax-rules ()
+			((define-simple-syntax (name . pattern) result)
+			 (define-syntax name (syntax-rules () ((name . pattern) result))))))))
+	
+
+(define-structure low-interrupt low-interrupt-interface
+  (open scheme
+	enumerated
+	bigbit
+	bitwise)
+  (files low-interrupt))
+
+;(define-structure test-package (export test-proc)
+;  (open scsh-regexp-package scheme)
+;  (begin (define (test-proc p)
+;	   (regexp-substitute p
+;			      (string-match "(foo)(.*)(bar)" "Hello foo Olin bar quux")
+;			      'post 3 1 2 'pre))))
+
+
+(define-structure scsh-threads
+  (export fork/thread
+	  fork/process
+	  wait/thread
+	  wait/process)
+  (open structure-refs
+	scheme)
+  (access scsh-level-0
+	  threads
+	  threads-internal)
+  (files threads))
+
+(define-structure dot-locking dot-locking-interface
+  (open scsh-level-0
+	scheme
+	let-opt
+	threads  ; sleep
+	random)
+  (files dot-locking))
+
+(define-structures ((syslog syslog-interface)
+		    (syslog-channels syslog-channels-interface))
+  (open scheme
+	define-record-types finite-types enum-sets
+	locks thread-fluids
+	external-calls
+	bitwise)
+  (files syslog))
+
