@@ -1,21 +1,20 @@
 ;;; A Scheme shell.
-;;; Copyright (c) 1992-1999 by Olin Shivers.
+;;; Copyright (c) 1992 by Olin Shivers.
 ;;; Copyright (c) 1994 by Brian D. Carlstrom.
-;;; See file COPYING.
 
 ;;; Call THUNK, then die.
 ;;; A clever definition in a clever implementation allows the caller's stack
 ;;; and dynamic env to be gc'd away, since this procedure never returns.
 
-(define (call-terminally thunk)
-  (with-continuation (loophole :escape #f)	; Bogus
-		     (lambda () (thunk) (exit 0))))
-  ;; Alternatively: (with-continuation #f thunk)
+;;;(define (call-terminally thunk)
+;;;  (with-continuation (lambda () #f) (lambda () (thunk) (exit 0))))
+;;;  ;; Alternatively: (with-continuation #f thunk)
 
 ;;; More portably, but less usefully:
-;;; (define (call-terminally thunk)
-;;;   (thunk)
-;;;   (exit 0))
+;;; New version of s48 requires with-continuation to take a continuation
+(define (call-terminally thunk)
+  (thunk)
+  (exit 0))
 
 ;;; Like FORK, but the parent and child communicate via a pipe connecting
 ;;; the parent's stdin to the child's stdout. This function side-effects
@@ -113,8 +112,226 @@
 
 
 
-;;; Environment stuff
+
+;;; Should be moved to somewhere else
+(define (with-lock lock thunk)
+  (dynamic-wind 
+   (lambda ()
+     (release-lock lock))
+   thunk
+   (lambda ()
+     (release-lock lock))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+
+
+;;; A resource is a part of the process state for which every thread
+;;; has its own value
+;;; uses the procedures:
+;;; (process-read-resource (-> 'X))
+;;; (process-set-resource ('X -> unspec))
+;;; (resource-eq? ('X 'X -> bool))
+
+;;; defines the procedures:
+;;; (initialize-resource  (-> unspec)) ; call on startup
+;;; (with-resource* ((-> 'X) -> 'X))
+;;; (with-resource-aligned* ((-> 'X) -> 'X))
+;;; (thread-read-resource (-> 'X))
+;;; (thread-set-resource ('X -> unspec))
+
+(define-syntax  make-process-resource
+  (syntax-rules 
+   ()
+   ((make-process-resource 
+     initialize-resource 
+     thread-read-resource thread-set-resource! thread-change-resource 
+     with-resource* with-resource-aligned* 
+     process-read-resource process-set-resource resource-eq?)
+    (begin 
+      (define *resource-cache* 'uninitialized)
+      (define resource-lock 'uninitialized)
+       
+      (define (initialize-resource)
+	(set! *resource-cache* (process-read-resource))
+	(set! $resource ;;; TODO The old thread-fluid will remain
+	      (make-preserved-thread-fluid
+	       (process-read-resource)))
+	(set! resource-lock (make-lock)))
+       
+      (define (cache-value)
+	*resource-cache*)
+       
+      ;; Actually do the syscall and update the cache
+      ;; assumes the resource lock obtained
+      (define (change-and-cache dir)
+	(process-set-resource dir)
+	(set! *resource-cache* (process-read-resource)))
+      
+      ;; The thread-specific resource: A thread fluid
+      
+      (define $resource 'empty-resource-value)
+       
+      (define (thread-read-resource) (thread-fluid $resource))
+      (define (thread-set-resource! dir) (set-thread-fluid! $resource dir))
+      (define (let-resource dir thunk)
+	(let-thread-fluid $resource dir thunk))
+       
+      (define (with-resource* dir thunk)
+	(let ((changed-dir #f))		
+	  (with-lock resource-lock
+		     (lambda ()
+		       (change-and-cache dir)
+		       (set! changed-dir (cache-value))))
+	  (let-resource changed-dir thunk)))
+       
+      ;; Align the value of the Unix resource with scsh's value.
+      ;; Since another thread could disalign, this call and
+      ;; any ensuring syscall that relies upon it should
+      ;; be "glued together" with the resource lock.
+      
+      (define (align-resource!)
+	(let ((dir (thread-read-resource)))
+	  (if (not (resource-eq? dir (cache-value)))
+	      (change-and-cache dir))))
+
+      (define (thread-change-resource dir)
+	(with-lock resource-lock
+		   (lambda ()
+		     (align-resource!)
+		     (change-and-cache dir)
+		     (thread-set-resource! (cache-value)))))
+
+      ;; For thunks that don't raise exceptions or throw to continuations,
+      ;; this is overkill & probably a little heavyweight for frequent use.
+      ;; But it is general.
+      ;;
+      ;; A less-general, more lightweight hack could be done just for
+      ;; syscalls.  We could probably dump the DYNAMIC-WINDs and build the
+      ;; rest of the pattern into one of the syscall-defining macros, or
+      ;; something.  
+      ;; Olin adds the following: the efficient way to do things is not
+      ;; with a dynamic wind or a lock. Just turn off interrupts, sync the
+      ;; resource, do the syscall, turn them back on.
+
+      (define (with-resource-aligned* thunk)
+	(dynamic-wind (lambda ()
+			(with-lock resource-lock
+				   align-resource!)) 
+		      thunk 
+		      values))
+
+      ;; example syscall
+      ;; (define (exported-delete-file fname)
+      ;;  (with-cwd-aligned (really-delete-file fname)))
+
+
+      (define resource-reinitializer
+	(make-reinitializer (lambda () (warn "calling resumer") (initialize-resource))))))))
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;; working directory per thread
+(make-process-resource 
+ initialize-cwd cwd thread-set-cwd! chdir with-cwd* with-cwd-aligned*
+ process-cwd process-chdir string=?)
+
+(initialize-cwd)
+
+;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;; umask per thread
+
+(make-process-resource 
+ initialize-umask umask thread-set-umask set-umask 
+ with-umask* with-umask-aligned*
+ process-umask set-process-umask =)
+
+(initialize-umask)
+
+(set-with-fs-context-aligned*! ; ensure S48 is aligned too
+ (lambda (thunk)
+   (with-cwd-aligned*
+    (lambda ()
+      (with-umask-aligned*
+       thunk)))))
+
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
+;;; Environment per thread
+
+(define-record env
+      envvec
+      alist)	; Corresponding alist
+
+(define-record-resumer type/env
+  (lambda (env)
+    (set-env:envvec env #f)))
+
+(define (env=? e1 e2)
+  (and (env:envvec e1)
+       (eq? (env:envvec e1) 
+	    (env:envvec e2))))
+
+(define-record envvec 
+  environ ;; char**
+  )
+
+(define (add-envvec-finalizer! envvec)
+  (add-finalizer! envvec envvec-finalizer))
+
+(define-exported-binding "envvec-record-type" type/envvec)
+(define-exported-binding "add-envvec-finalizer!" add-envvec-finalizer!)
+
+(define (envvec-finalizer envvec)
+  (%free-env envvec))
+
+(define (environ**-read)
+  (let ((alist.envvec (environ-env->alist)))
+    (make-env (cdr alist.envvec) (car alist.envvec))))
+
+(define (environ**-set env)
+   (if (env:envvec env)
+       (%align-env (env:envvec env))
+       (set-env:envvec env (envvec-alist->env (env:alist env)))))
+
+(define (getenv var)
+  (let* ((env (thread-read-env))
+	 (res (assoc var (env:alist env))))
+    (if res (cdr res) res)))
+
+(define (env->alist)
+  (env:alist (thread-read-env)))
+
+(define (setenv var val)
+  (let* ((env (thread-read-env))
+	 (alist (alist-update 
+		 var 
+		 val 
+		 (fold cons '() (env:alist env)))))
+    (thread-set-env! 
+     (make-env 
+      #f
+      alist
+      ))))
+
+(define (alist->env alist)
+  (thread-set-env!
+   (make-env
+    #f
+    alist)))
+
+(define (with-env* alist-delta thunk)
+  (let ((new-env (fold (lambda (key/val alist)
+			  (alist-update (car key/val) (cdr key/val) alist))
+			(env->alist)
+			alist-delta)))
+    (with-total-env* new-env thunk)))
+
+(define (with-total-env* alist thunk)
+  (with-env-internal* (make-env #f alist) thunk))
+
+(make-process-resource install-env thread-read-env thread-set-env! 
+		       useless-set-env
+		       with-env-internal* with-env-aligned* 
+		       environ**-read environ**-set env=?)
 
 ;;; These two functions are obsoleted by the more general INFIX-SPLITTER and
 ;;; JOIN-STRINGS functions. However, we keep SPLIT-COLON-LIST defined
@@ -168,16 +385,6 @@
 				       (cons key/val ans))))
 		 ans))))
 
-;; Tail-recursive loops suck.
-;; (define (alist-compress alist)
-;;   (loop (initial (ans '()))
-;;	   (for key/val in alist)
-;;   
-;;	   (when (not (assoc (car key/val) ans)))
-;;	   (next (ans (cons key/val ans)))
-;;   
-;;	   (result (reverse ans))))
-
 (define (add-before elt before list)
   (let rec ((list list))
     (if (pair? list)
@@ -204,63 +411,17 @@
 	    #f))			; AFTER doesn't appear in LIST.
       (cons elt list))) 
 
-;;; Or, just say...
-;;; (reverse (add-before elt after (reverse list)))
-
-(define (with-env* alist-delta thunk)
-  (let* ((old-env #f)
-	 (new-env (fold (lambda (key/val alist)
-			  (alist-update (car key/val) (cdr key/val) alist))
-			(env->alist)
-			alist-delta)))
-    (dynamic-wind
-      (lambda ()
-	(set! old-env (env->alist))
-	(alist->env new-env))
-      thunk
-      (lambda ()
-	(set! new-env (env->alist))
-	(alist->env old-env)))))
-
-(define (with-total-env* alist thunk)
-  (let ((old-env (env->alist)))
-    (dynamic-wind
-      (lambda ()
-	(set! old-env (env->alist))
-	(alist->env alist))
-      thunk
-      (lambda ()
-	(set! alist (env->alist))
-	(alist->env old-env)))))
-
-
-(define (with-cwd* dir thunk)
-  (let ((old-wd #f))
-    (dynamic-wind
-      (lambda ()
-	(set! old-wd (cwd))
-	(chdir dir))
-      thunk
-      (lambda ()
-	(set! dir (cwd))
-	(chdir old-wd)))))
-
-(define (with-umask* mask thunk)
-  (let ((old-mask #f))
-    (dynamic-wind
-      (lambda ()
-	(set! old-mask (umask))
-	(set-umask mask))
-      thunk
-      (lambda ()
-	(set! mask (umask))
-	(set-umask old-mask)))))
-
 ;;; Sugar:
 
 (define-simple-syntax (with-cwd dir . body)
   (with-cwd* dir (lambda () . body)))
 
+(define-simple-syntax (with-cwd-aligned body ...)
+  (with-cwd-aligned* (lambda () body ...)))
+
+(define-simple-syntax (with-umask-aligned body ...)
+  (with-cwd-aligned* (lambda () body ...)))
+
 (define-simple-syntax (with-umask mask . body)
   (with-umask* mask (lambda () . body)))
 
@@ -270,7 +431,7 @@
 (define-simple-syntax (with-total-env env . body)
   (with-total-env* `env (lambda () . body)))
 
-
+
 (define (call/temp-file writer user)
   (let ((fname #f))
     (dynamic-wind
@@ -283,7 +444,7 @@
 
 ;;; Create a new temporary file and return its name.
 ;;; The optional argument specifies the filename prefix to use, and defaults
-;;; to "/usr/tmp/<pid>.", where <pid> is the current process' id. The procedure
+;;; to "/tmp/<pid>.", where <pid> is the current process' id. The procedure
 ;;; scans through the files named <prefix>0, <prefix>1, ... until it finds a
 ;;; filename that doesn't exist in the filesystem. It creates the file with 
 ;;; permission #o600, and returns the filename.
@@ -300,9 +461,22 @@
 	       (list (string-append (constant-format-string (car maybe-prefix))
 				    ".~a"))))))
 
-(define *temp-file-template*
-  (make-fluid (string-append "/usr/tmp/" (number->string (pid)) ".~a")))
+(define (initial-temp-file)
+  (let ((tmpdir (getenv "TMPDIR")))
+    (string-append
+     (if tmpdir
+	 tmpdir
+	 "/var/tmp")
+     "/"
+     (number->string (pid))
+     "~a")))
 
+(define *temp-file-template* (make-fluid 'not-initialized-temp-file-template))
+
+(define temp-file-reinitializer 
+  (make-reinitializer 
+   (lambda ()
+     (set-fluid! *temp-file-template* (initial-temp-file)))))
 
 (define (temp-file-iterate maker . maybe-template)
   (let ((template (:optional maybe-template (fluid *temp-file-template*))))
@@ -599,12 +773,12 @@
 (define (stdio->stdports)
   (set-current-input-port!  (fdes->inport 0))
   (set-current-output-port! (fdes->outport 1))
-  (set-error-output-port!   (fdes->outport 2)))
+  (set-current-error-port!   (fdes->outport 2)))
 
 (define (with-stdio-ports* thunk)
   (with-current-input-port (fdes->inport 0)
     (with-current-output-port (fdes->outport 1)
-      (with-error-output-port (fdes->outport 2)
+      (with-current-error-port (fdes->outport 2)
 	(thunk)))))
 
 (define-simple-syntax (with-stdio-ports body ...)
@@ -614,7 +788,7 @@
 (define (stdports->stdio)
   (dup (current-input-port)  0)
   (dup (current-output-port) 1)
-  (dup (error-output-port)   2))
+  (dup (current-error-port)   2))
 
 
 ;;; Command-line argument access
@@ -679,7 +853,13 @@
 		    
 (define (exec/env prog env . arglist)
   (flush-all-ports)
-  (%exec prog (cons prog arglist) env))
+  (with-env-aligned* 
+   (lambda ()
+     (with-cwd-aligned*
+      (lambda ()
+	(with-umask-aligned*
+	 (lambda ()
+	   (%exec prog (cons prog arglist) env))))))))
 
 ;(define (exec-path/env prog env . arglist)
 ;  (cond ((exec-path-search (stringify prog) (fluid exec-path-list)) =>
@@ -693,21 +873,27 @@
 
 (define (exec-path/env prog env . arglist)
   (flush-all-ports)
-  (let ((prog (stringify prog)))
-    (if (string-index prog #\/)
-
-	;; Contains a slash -- no path search.
-	(%exec prog (cons prog arglist) env)
-
-	;; Try each directory in PATH-LIST.
-	(let ((argv (list->vector (cons prog (map stringify arglist)))))
-	  (for-each (lambda (dir)
-		      (let ((binary (string-append dir "/" prog)))
-			(%%exec/errno binary argv env)))
-		    (fluid exec-path-list)))))
-
-    (error "No executable found." prog arglist))
-	 
+  (with-env-aligned*
+   (lambda ()
+     (with-cwd-aligned*
+      (lambda ()
+	(with-umask-aligned*
+	 (lambda ()
+	   (let ((prog (stringify prog)))
+	     (if (string-index prog #\/)
+		 
+		 ;; Contains a slash -- no path search.
+		 (%exec prog (cons prog arglist) env)
+		 
+		 ;; Try each directory in PATH-LIST.
+		 (let ((argv (list->vector (cons prog (map stringify arglist)))))
+		   (for-each (lambda (dir)
+			       (let ((binary (string-append dir "/" prog)))
+				 (%%exec binary argv env)))
+			     (fluid exec-path-list)))))
+	   
+	   (error "No executable found." prog arglist))))))))
+   
 (define (exec-path prog . arglist)
   (apply exec-path/env prog #t arglist))
 
@@ -724,29 +910,35 @@
 (define (%fork . maybe-thunk)
   (really-fork #f maybe-thunk))
 
+
 (define (really-fork clear-interactive? maybe-thunk)
-  ((with-enabled-interrupts 0
+  (with-env-aligned* ; not neccessary here but doing it on exec
+                     ; genereates no cache in the parent
+   (lambda ()
+   (((structure-ref interrupts with-interrupts-inhibited) (lambda ()
      (let ((pid (%%fork)))
        (if (zero? pid)				
 
 	   ;; Child
 	   (lambda ()	; Do all this outside the WITH-INTERRUPTS.
-	     (set! reaped-procs '())
-	     (if clear-interactive?
+;	     (set! reaped-procs '())
+
+	     ;;; There is no session if parent was started in batch-mode
+	     (if (and (session-started?) clear-interactive?)
 		 (set-batch-mode?! #t))	; Children are non-interactive.
 	     (and (pair? maybe-thunk)
 		  (call-terminally (car maybe-thunk))))
 
 	   ;; Parent
 	   (let ((proc (new-child-proc pid)))
-	     (lambda () proc)))))))
+	     (lambda () proc))))))))))
 
 
 (define (exit . maybe-status)
   (flush-all-ports)
   (exit/errno (:optional  maybe-status 0))
   (display "The evil undead walk the earth." 2)
-  (error "(exit) returned."))
+  (if #t (error "(exit) returned.")))
 
 
 ;;; The classic T 2.0 primitive.
@@ -757,9 +949,11 @@
 ;;; Low-level init absolutely required for any scsh program.
 
 (define (init-scsh-hindbrain relink-ff?)
-  (if relink-ff? (lookup-all-externals)) ; Re-link C calls.
-  (init-fdports!)
-  (%install-unix-scsh-handlers))
+  (if #t (error "call to init-scsh-hindbrain which is dead"))
+;  (if relink-ff? (lookup-all-externals)) ; Re-link C calls.
+;  (init-fdports!)
+;  (%install-unix-scsh-handlers)
+)
 
 
 ;;; Some globals:
@@ -781,3 +975,4 @@
 
 ; SIGTSTP blows s48 away. ???
 (define (suspend) (signal-process 0 signal/stop))
+
