@@ -15,8 +15,6 @@
 
 #include "cstuff.h"
 #include "fdports.h"	/* Accessors for Scheme I/O port internals. */
-#include "fdports1.h"	/* Import fdes2fstar(). */
-#include "machine/stdio_dep.h"	/* Import stdio buf-peeking ops. */
 
 /* Make sure our exports match up w/the implementation: */
 #include "select1.h"
@@ -24,10 +22,10 @@
 /* the traditional sleazy max non-function. */
 #define max(a,b) (((a) > (b)) ? (a) : (b))
 
-extern FILE *fdes2fstar(int fd);
+extern int errno;
 
 static void or2_fdset(fd_set *x, fd_set *y, int max_elt);
-static int copyback_fdvec(scheme_value portvec, fd_set *fdset);
+static int copyback_fdvec(s48_value portvec, fd_set *fdset);
 
 /* RVEC, WVEC, and EVEC are Scheme vectors of integer file descriptors,
 ** I/O ports, and #f's. NSECS is an integer timeout value, or #f for
@@ -35,8 +33,8 @@ static int copyback_fdvec(scheme_value portvec, fd_set *fdset);
 ** passed pointers. Return 0 for OK, otherwise error is in errno.
 */
 
-int do_select(scheme_value rvec, scheme_value wvec,
-	      scheme_value evec, scheme_value nsecs,
+int do_select(s48_value rvec, s48_value wvec,
+	      s48_value evec, s48_value nsecs,
 	      fd_set *rset_ans, fd_set *wset_ans, fd_set *eset_ans)
 {
     struct timeval timeout, *tptr;
@@ -50,74 +48,37 @@ int do_select(scheme_value rvec, scheme_value wvec,
     FD_ZERO(&rset_bufrdy);	FD_ZERO(&wset_bufrdy);	FD_ZERO(&eset_bufrdy);
 
     /* Scan the readvec elts. */
-    nelts = VECTOR_LENGTH(rvec);
+    nelts = S48_VECTOR_LENGTH(rvec);
     for(i=nelts; --i >= 0; ) {
-	scheme_value elt = VECTOR_REF(rvec,i);
+	s48_value elt = S48_VECTOR_REF(rvec,i);
 	int fd;
 
-	if( FIXNUMP(elt) ) {			    /* It's an integer fdes. */
-	    fd = EXTRACT_FIXNUM(elt);
-	    FD_SET(fd, rset_ans);
-	    }
-
-	else if( elt != SCHFALSE ) {		    /* It better be a port. */
-	    FILE *f;
-	    scheme_value data = *Port_PortData(elt);
-	    fd = EXTRACT_FIXNUM(*PortData_Fd(data));
-	    f = fdes2fstar(fd);
-	    if( !f ) return -1;
-	    else if( *PortData_Peek(data) != SCHFALSE /* Port has a peekchar */
-		     || !ibuf_empty(f) ) { 	     /* Stdio buf has chars. */
-		FD_SET(fd, &rset_bufrdy);
-		rbuf_rdy = 1; /* Hit. */
-		}
-	    else FD_SET(fd, rset_ans);		  /* No buffered data. */
-	    }
+	fd = s48_extract_fixnum(elt);
+	FD_SET(fd, rset_ans);
 
 	max_fd = max(max_fd, fd);
 	}
 
     /* Scan the writevec elts. */
-    nelts = VECTOR_LENGTH(wvec);
+    nelts = S48_VECTOR_LENGTH(wvec);
     for(i=nelts; --i >= 0; ) {
-	scheme_value elt = VECTOR_REF(wvec,i);
+	s48_value elt = S48_VECTOR_REF(wvec,i);
 	int fd;
 
-	if( FIXNUMP(elt) ) {			    /* It's an integer fdes. */
-	    fd = EXTRACT_FIXNUM(elt);
-	    FD_SET(fd, wset_ans);
-	    }
-
-	else if( elt != SCHFALSE ) {		    /* It better be a port. */
-	    FILE *f;
-	    fd = EXTRACT_FIXNUM(*PortFd(elt));
-	    f = fdes2fstar(fd);
-	    if( !f ) return -1;
-	    else if( !obuf_full(f) ) {		  /* I/O buf has room. */
-		FD_SET(fd, &wset_bufrdy);		
-		wbuf_rdy = 1; /* Hit. */
-		}
-	    else FD_SET(fd, wset_ans);		  /* No room. */
-	    }
+	fd = s48_extract_fixnum(elt);
+	FD_SET(fd, wset_ans);
 
 	max_fd = max(max_fd, fd);
 	}
 
     /* Scan the exception-vec elts. */
-    nelts = VECTOR_LENGTH(evec);
+    nelts = S48_VECTOR_LENGTH(evec);
     for(i=nelts; --i >= 0; ) {
-	scheme_value elt = VECTOR_REF(evec,i);
+	s48_value elt = S48_VECTOR_REF(evec,i);
 	int fd;
 
-	if( FIXNUMP(elt) ) {			/* It's an integer fdes. */
-	    fd = EXTRACT_FIXNUM(elt);
-	    FD_SET(fd, rset_ans);
-	    }
-
-	else if( elt != SCHFALSE ) {		    /* It better be a port. */
-	    fd = EXTRACT_FIXNUM(*PortFd(elt));
-	    FD_SET(fd, rset_ans);
-	    }
+	fd = s48_extract_fixnum(elt);
+	FD_SET(fd, eset_ans);
 
 	max_fd = max(max_fd, fd);
 	}
@@ -128,15 +89,15 @@ int do_select(scheme_value rvec, scheme_value wvec,
 	timeout.tv_usec = 0;
 	tptr = &timeout;
 	}
-    else if ( FIXNUMP(nsecs) ) {
-	timeout.tv_sec = EXTRACT_FIXNUM(nsecs);	/* Wait n seconds. */
+    else if ( S48_FIXNUM_P(nsecs) ) {
+	timeout.tv_sec = s48_extract_fixnum(nsecs);	/* Wait n seconds. */
 	timeout.tv_usec = 0;
 	tptr = &timeout;
 	}
     else tptr = NULL;				/* #f => Infinite wait. */
 
     /* select1() is defined in sysdep.h -- bogus compatibility macro. */
-    nfound = select1(max_fd+1, rset_ans, wset_ans, eset_ans, tptr); /* Do it.*/
+    nfound = select(max_fd+1, rset_ans, wset_ans, eset_ans, tptr); /* Do it.*/
 
     /* EINTR is not an error return if we have hits on buffered ports
     ** to report.
@@ -172,16 +133,17 @@ static void or2_fdset(fd_set *x, fd_set *y, int max_elt)
 ** Scan over the vector, and copy any elt whose file descriptor is in FDSET
 ** to the front of the vector. Return the number of elts thus copied.
 */
-static int copyback_fdvec(scheme_value portvec, fd_set *fdset)
+static int copyback_fdvec(s48_value portvec, fd_set *fdset)
 {
-    int vlen = VECTOR_LENGTH(portvec);
+    int vlen = S48_VECTOR_LENGTH(portvec);
     int i, j=0;
     for( i = -1; ++i < vlen; ) {
-	scheme_value elt = VECTOR_REF(portvec, i);
-	int fd = EXTRACT_FIXNUM((FIXNUMP(elt)) ? elt : *PortFd(elt));
+	s48_value elt = S48_VECTOR_REF(portvec, i);
+	int fd = s48_extract_fixnum((S48_FIXNUM_P(elt)) ? elt : (1 / 0));
+	//JMG *PortFd(elt));
 	if( FD_ISSET(fd,fdset) ) {
 	    FD_CLR(fd,fdset);	/* In case luser put elt in multiple times. */
-	    VECTOR_REF(portvec, j) = elt;
+	    S48_VECTOR_SET(portvec, j, elt);
 	    j++;
 	    }
 	}
@@ -193,20 +155,20 @@ static int copyback_fdvec(scheme_value portvec, fd_set *fdset)
 ** Return count of active elements.
 */
 
-static int clobber_inactives(scheme_value portvec, fd_set *fdset)
+static int clobber_inactives(s48_value portvec, fd_set *fdset)
 {
     int count = 0;
-    int i = VECTOR_LENGTH(portvec);
+    int i = S48_VECTOR_LENGTH(portvec);
 
     while( --i >= 0 ) {
-	scheme_value elt = VECTOR_REF(portvec, i);
-	if( elt != SCHFALSE ) {
-	    int fd = EXTRACT_FIXNUM((FIXNUMP(elt)) ? elt : *PortFd(elt));
+	s48_value elt = S48_VECTOR_REF(portvec, i);
+	if( elt != S48_FALSE ) {
+	  int fd = s48_extract_fixnum((S48_FIXNUM_P(elt)) ? elt : (1/0)); //JMG *PortFd(elt));
 	    if( FD_ISSET(fd,fdset) ) {
 		FD_CLR(fd,fdset); /* In case luser put elt in multiple times. */
 		++count;
 		}
-	    else VECTOR_REF(portvec, i) = SCHFALSE; /* Clobber. */
+	    else S48_VECTOR_SET(portvec, i, S48_FALSE); /* Clobber. */
 	    }
 	}
     return count;
@@ -221,21 +183,21 @@ static int clobber_inactives(scheme_value portvec, fd_set *fdset)
 ** Return error indicator & number of hits for each vector.
 */
 
-scheme_value select_copyback(scheme_value rvec, scheme_value wvec,
-			     scheme_value evec, scheme_value nsecs,
+s48_value select_copyback(s48_value rvec, s48_value wvec,
+			     s48_value evec, s48_value nsecs,
 			     int *r_numrdy, int *w_numrdy, int *e_numrdy)
 {
     fd_set rset, wset, eset;
 
     if( do_select(rvec, wvec, evec, nsecs, &rset, &wset, &eset) ) {
 	*r_numrdy = *w_numrdy = *e_numrdy = 0;
-	return ENTER_FIXNUM(errno);
+	return s48_enter_fixnum(errno);
 	}
     
     *r_numrdy = copyback_fdvec(rvec, &rset);
     *w_numrdy = copyback_fdvec(wvec, &wset);
     *e_numrdy = copyback_fdvec(evec, &eset);
-    return SCHFALSE;
+    return S48_FALSE;
     }
 
 
@@ -243,19 +205,19 @@ scheme_value select_copyback(scheme_value rvec, scheme_value wvec,
 ** return error indicator & number of hits for each vector.
 */
 
-scheme_value select_filter(scheme_value rvec, scheme_value wvec,
-			   scheme_value evec, scheme_value nsecs,
+s48_value select_filter(s48_value rvec, s48_value wvec,
+			   s48_value evec, s48_value nsecs,
 			   int *r_numrdy, int *w_numrdy, int *e_numrdy)
 {
     fd_set rset, wset, eset;
 
     if( do_select(rvec, wvec, evec, nsecs, &rset, &wset, &eset) ) {
 	*r_numrdy = *w_numrdy = *e_numrdy = 0;
-	return ENTER_FIXNUM(errno);
+	return s48_enter_fixnum(errno);
 	}
     
     *r_numrdy = clobber_inactives(rvec, &rset);
     *w_numrdy = clobber_inactives(wvec, &wset);
     *e_numrdy = clobber_inactives(evec, &eset);
-    return SCHFALSE;
+    return S48_FALSE;
     }
