@@ -2,10 +2,7 @@
 ** Copyright (c) 1994 by Olin Shivers.
 */
 
-/* WARNING: THIS FILE HAS CODE THAT DEPENDS ON 32-BIT ARCHITECTURES.
-** This code is so marked.
-**
-** The source code is also conditionalised by three #ifdef feature macros:
+/* The source code is conditionalised by three #ifdef feature macros:
 ** HAVE_TZNAME
 **   The char *tzname[2] global variable is POSIX. Everyone provides
 **   it...except some "classic" versions of SunOS that we still care about
@@ -49,16 +46,16 @@
 #include "cstuff.h"
 #include "time1.h"	/* Make sure the .h interface agrees with the code. */
 
+#include <stdio.h>     // JMG debug
+
 extern char **environ;
 
 /* To work in the UTC time zone, do "environ = utc_env;". */
 static char *utc_env[] = {"TZ=UCT0", 0};
 
-#ifdef HAVE_TZNAME
-#ifndef __CYGWIN__
+#if defined(HAVE_TZNAME) && !defined(CYGWIN)
 extern char *tzname[];	/* Why isn't this defined in time.h? */
 #endif
-#endif
 
 /* These two functions allow you to temporarily override
 ** the current time zone with one of your choice. make_newenv()
@@ -71,14 +68,21 @@ extern char *tzname[];	/* Why isn't this defined in time.h? */
 ** On error, make_newenv returns NULL.
 */
 
-static char **make_newenv(scheme_value zone, char *newenv[2])
+static char **make_newenv(s48_value zone, char *newenv[2])
 {	
-    int zonelen = STRING_LENGTH(zone);
+    int zonelen = S48_STRING_LENGTH(zone);
     char **oldenv = environ,
          *tz = Malloc(char, 4+zonelen);
+    
+    char * extracted_zone = s48_extract_string(zone);
+    
+    // s48_value temp;
     if( !tz ) return NULL;
     strcpy(tz, "TZ=");
-    strncpy(tz+3, &STRING_REF(zone,0), zonelen);
+
+    //    temp = S48_STRING_REF(zone,0); //JMG
+    // strncpy(tz+3, &temp, zonelen);
+    strncpy(tz+3, extracted_zone, zonelen);
     tz[zonelen+3] = '\0';
     newenv[0] = tz;
     newenv[1] = NULL;
@@ -97,54 +101,67 @@ static void revert_env(char **old_env)
 
 /*****************************************************************************/
 
-/* Sux because it's dependent on 32-bitness. */
-#define hi8(i)  (((i)>>24) & 0xff)
-#define lo24(i) ((i) & 0xffffff)
-#define comp8_24(hi, lo) (((hi)<<24) + (lo))
 
-scheme_value scheme_time(int *hi_secs, int *lo_secs)
+
+
+s48_value scheme_time()
 {
     time_t t;
     errno = 0;
     t = time(NULL);
-    if( t == -1 && errno ) return ENTER_FIXNUM(errno);
-    *hi_secs = hi8(t);
-    *lo_secs = lo24(t);
-    return SCHFALSE;
-    }
+    if( t == -1 && errno ) s48_raise_os_error (errno);
+    return s48_enter_integer (t);
+}
+
+// should be part of the FFI interface
+s48_value s48_list_11 (s48_value e1, s48_value e2, s48_value e3, 
+		       s48_value e4, s48_value e5, s48_value e6, 
+		       s48_value e7, s48_value e8, s48_value e9, 
+		       s48_value e10, s48_value e11)
+{
+  return 
+    s48_cons (e1, s48_cons (e2, s48_cons (e3, s48_cons (e4, s48_cons 
+	(e5, s48_cons (e6, s48_cons (e7, s48_cons (e8, s48_cons (e9, 
+     s48_cons (e10, s48_cons (e11, S48_NULL)))))))))));
+}
+
 
 /* Zone:
 **   #f		Local time
 **   int	Offset from GMT in seconds.
 **   string	Time zone understood by OS.
 */
-scheme_value time2date(int hi_secs, int lo_secs, scheme_value zone,
-		       int *sec, int *min, int *hour,
+s48_value time2date(s48_value sch_t, s48_value sch_zone)
+     /*		       int *sec, int *min, int *hour,
 		       int *mday, int *month, int *year,
 		       const char **tz_name, int *tz_secs,
 		       int *summer,
 		       int *wday, int *yday)
+     */
 {
-    time_t t = comp8_24(hi_secs, lo_secs);
-    struct tm d;
+   struct tm d;
+   time_t t = s48_extract_integer(sch_t);
+   s48_value sch_tz_name = S48_UNSPECIFIC;
+   s48_value sch_tz_secs = S48_UNSPECIFIC;
 
-    if( FIXNUMP(zone) ) {			/* Offset from GMT in secs. */
-	int offset = EXTRACT_FIXNUM(zone);
-	t += EXTRACT_FIXNUM(zone);
+   if( S48_FIXNUM_P(sch_zone) ) {		/* Offset from GMT in secs. */
+	int offset = s48_extract_fixnum(sch_zone);
+	t += s48_extract_fixnum(sch_zone);
 	d = *gmtime(&t);
-	*tz_name = NULL;
-	*tz_secs = offset;
+	sch_tz_name = s48_enter_string("");
+	sch_tz_secs = s48_enter_fixnum (offset);
 	}
     else {
 	char *newenv[2], **oldenv = NULL;
 
-	if( STRINGP(zone) ) {			/* Time zone */
-	    oldenv = make_newenv(zone, newenv); 	/* Install new TZ. */
-	    if( !oldenv ) return ENTER_FIXNUM(errno);	/* Error installing. */
-	    d = *localtime(&t);			   	/* Do it. */
-	    }
+	if( S48_STRING_P(sch_zone) ) {			/* Time zone */
+	  
+	  oldenv = make_newenv(sch_zone, newenv); 	/* Install new TZ. */
+	  if( !oldenv ) s48_raise_os_error_2(errno, sch_t, sch_zone);
+	  d = *localtime(&t);			   	/* Do it. */
+	}
 	else					/* Local time */
-	    d = *localtime(&t);
+	  d = *localtime(&t);
 
 	/* This little chunk of code copies the calculated time zone into
 	** a malloc'd buffer and assigns it to *tz_name. It's a little
@@ -163,37 +180,44 @@ scheme_value time2date(int hi_secs, int lo_secs, scheme_value zone,
 	  char *zone = tzname[d.tm_isdst];
 #endif
 	  char *newzone = Malloc(char, 1+strlen(zone));
-	  *tz_name = newzone;
-	  if( newzone ) strcpy(newzone, zone);
+	  if( newzone ){
+	    strcpy(newzone, zone);
+	    sch_tz_name = s48_enter_string (newzone);
+	  }
 	  else error = errno;
 	  
-	  if( oldenv ) revert_env(oldenv);		/* Revert TZ & env. */
+	  if( oldenv ) revert_env(oldenv); 		/* Revert TZ & env. */
 
-	  if( !newzone ) return ENTER_FIXNUM(error);
+	  if( !newzone ) s48_raise_os_error_2(error, sch_t, sch_zone);
 	  }
 
 	/* Calculate the time-zone offset in seconds from UTC. */
 #ifdef HAVE_GMTOFF	
-	*tz_secs = d.tm_gmtoff;
+	sch_tz_secs = s48_enter_fixnum (d.tm_gmtoff);
+	
 #else
-	{ struct tm dcopy = d;
-	  char **oldenv = environ;			/* Set TZ to UTC     */
+	{ char **oldenv = environ;			/* Set TZ to UTC     */
 	  environ=utc_env;				/* time temporarily. */
 	  tzset(); /* NetBSD, SunOS POSIX-noncompliance requires this. */
-	  dcopy.tm_isdst = 0;
-	  *tz_secs = mktime(&dcopy) - t;	/* mktime() may mung dcopy. */
+	  sch_tz_secs = s48_enter_fixnum (mktime(&d) - t);
 	  environ=oldenv;
+	  
 	  }
 #endif
 	}
-
-    *sec  = d.tm_sec;	*min   = d.tm_min;	*hour   = d.tm_hour;
-    *mday = d.tm_mday;	*month = d.tm_mon;	*year   = d.tm_year;
-    *wday = d.tm_wday;	*yday  = d.tm_yday;	*summer = d.tm_isdst;
-    return SCHFALSE;
+   return s48_list_11 (s48_enter_fixnum (d.tm_sec),
+		       s48_enter_fixnum (d.tm_min),
+		       s48_enter_fixnum (d.tm_hour),
+		       s48_enter_fixnum (d.tm_mday),
+		       s48_enter_fixnum (d.tm_mon),
+		       s48_enter_fixnum (d.tm_year),
+		       sch_tz_name,
+		       sch_tz_secs,
+		       d.tm_isdst ? S48_TRUE : S48_FALSE,
+		       s48_enter_fixnum (d.tm_wday),
+		       s48_enter_fixnum (d.tm_yday));	
 }
 
-
 /* Oops
 ** There's a fundamental problem with the Posix mktime() function used below
 ** -- it's error return value (-1) is also a valid return value, for date
@@ -206,20 +230,23 @@ scheme_value time2date(int hi_secs, int lo_secs, scheme_value zone,
 ** Who designed this interface?
 */
 
-scheme_value date2time(int sec, int min, int hour,
-		       int mday, int month, int year,
-		       scheme_value tz_name, scheme_value tz_secs,
-		       int summer,
-		       int *hi_secs, int *lo_secs)
+s48_value date2time(s48_value sec, s48_value min, s48_value hour,
+		    s48_value mday, s48_value month, s48_value year,
+		    s48_value tz_name, s48_value tz_secs,
+		    s48_value summer)
 {
     time_t t;
     struct tm d;
 
-    d.tm_sec  = sec;	d.tm_min  = min;	d.tm_hour  = hour;
-    d.tm_mday = mday;	d.tm_mon  = month;	d.tm_year  = year;
-    d.tm_wday = 0;	d.tm_yday = 0;		d.tm_isdst = summer;
+    d.tm_sec  = s48_extract_fixnum (sec); 
+    d.tm_min  = s48_extract_fixnum (min);
+    d.tm_hour = s48_extract_fixnum (hour);
+    d.tm_mday = s48_extract_fixnum (mday);
+    d.tm_mon  = s48_extract_fixnum (month);
+    d.tm_year = s48_extract_fixnum (year);
+    d.tm_wday = 0;	d.tm_yday = 0;	      
 
-    if( FIXNUMP(tz_secs) ) {		/* Offset from GMT in seconds. */
+    if( S48_FIXNUM_P(tz_secs) ) {		/* Offset from GMT in seconds. */
 	char **oldenv = environ;			/* Set TZ to UTC     */
 	environ = utc_env;				/* time temporarily. */
 	tzset(); /* NetBSD, SunOS POSIX-noncompliance requires this. */
@@ -227,34 +254,42 @@ scheme_value date2time(int sec, int min, int hour,
 	errno = 0;
 	t = mktime(&d);
 	/* t == -1 => you probably have an error. */
-	if( t == -1 ) return ENTER_FIXNUM(errno ? errno : -1);
-	t -= EXTRACT_FIXNUM(tz_secs);
+	if ((t == -1) && (errno != 0))
+	  // Sorry, we only have a version with 5 arguments...
+	  s48_raise_os_error_5 (errno, sec, min, hour, mday, month);
+	t -= s48_extract_fixnum(tz_secs);
 	environ = oldenv;
 	}
 
-    else if( STRINGP(tz_name) ) {	/* Time zone */
+    /* ### Note that we *still* don't implement the manual paragraph
+       with "When calcultating with time-zones, the date's SUMMER?
+       field is used to resolve ambiguities. */
+    else if( S48_STRING_P(tz_name) ) {	/* Time zone */
 	char *newenv[2];
 	char **oldenv = make_newenv(tz_name, newenv);
-	if( !oldenv ) return ENTER_FIXNUM(errno);
+	if( !oldenv ) return s48_enter_fixnum(errno);
 	tzset(); /* NetBSD, SunOS POSIX-noncompliance requires this. */
 	errno = 0;
+
+	d.tm_isdst = -1;
 	t = mktime(&d);
-	if( t == -1 ) return ENTER_FIXNUM(errno ? errno : -1);
+	if ((t == -1) && (errno != 0))
+	  s48_raise_os_error_5 (errno, sec, min, hour, mday, month);
 	revert_env(oldenv);
 	}
 
     else {				/* Local time */
 	tzset(); /* NetBSD, SunOS POSIX-noncompliance requires this. */
 	errno = 0;
+	d.tm_isdst = -1;
 	t = mktime(&d);
-	if( t == -1) return ENTER_FIXNUM(errno ? errno : -1);
-	}
-
-    *hi_secs = hi8(t);
-    *lo_secs = lo24(t);
-    return SCHFALSE;
+	if ((t == -1) && (errno != 0))
+	  s48_raise_os_error_5 (errno, sec, min, hour, mday, month);
     }
 
+    return s48_enter_integer(t);
+}
+
 
 /* WARNING WARNING WARNING WARNING WARNING WARNING WARNING WARNING
 **
@@ -284,13 +319,14 @@ scheme_value date2time(int sec, int min, int hour,
 ** Professional programmers sacrifice their pride that others may live.
 ** Why me? Why Unix?
 */
-scheme_value format_date(const char *fmt, int sec, int min, int hour,
-			 int mday, int month, int year,
-			 scheme_value tz, int summer,
-			 int week_day, int year_day,
-			 const char **ans)
+s48_value format_date(s48_value sch_fmt, s48_value sch_sec, s48_value sch_min,
+		      s48_value sch_hour, s48_value sch_mday, 
+		      s48_value sch_month, s48_value sch_year,
+		      s48_value sch_tz, s48_value sch_summer,
+		      s48_value sch_week_day, s48_value sch_year_day)
 {
     struct tm d;
+    char *fmt = s48_extract_string(sch_fmt);
     int fmt_len = strlen(fmt);
     char *fmt2 = Malloc(char, 2+2*fmt_len); /* 1 extra for prefixed "x" char.*/
     int target_len = 1; 	/* 1 for the prefixed "x" char. Ugh. */
@@ -299,13 +335,23 @@ scheme_value format_date(const char *fmt, int sec, int min, int hour,
     const char *p;
     char *newenv[2], **oldenv = NULL;
     int result_len;
-    
-    *ans = NULL;	/* In case we error out. */
-    if( !fmt2 ) return ENTER_FIXNUM(errno);
+    s48_value sch_ans = S48_UNSPECIFIC;
 
-    d.tm_sec  = sec;		d.tm_min  = min;	d.tm_hour  = hour;
-    d.tm_mday = mday;		d.tm_mon  = month;	d.tm_year  = year;
-    d.tm_wday = week_day;	d.tm_yday = year_day;	d.tm_isdst = summer;
+    if( !fmt2 ) s48_raise_os_error_1(errno, sch_fmt); 
+
+    d.tm_sec  = s48_extract_fixnum(sch_sec);
+    d.tm_min  = s48_extract_fixnum(sch_min);
+    d.tm_hour  = s48_extract_fixnum(sch_hour);
+    d.tm_mday = s48_extract_fixnum(sch_mday);
+    d.tm_mon  = s48_extract_fixnum(sch_month);
+    d.tm_year  = s48_extract_fixnum(sch_year);
+    d.tm_wday = s48_extract_fixnum(sch_week_day);
+    d.tm_yday = s48_extract_fixnum(sch_year_day);
+    d.tm_isdst = (S48_EQ_P (sch_summer, S48_FALSE)) ? 0 : 1;
+
+#ifdef HAVE_TM_ZONE
+    d.tm_zone = s48_extract_string(sch_tz); /* FreeBSD's strftime reads this */
+#endif
 
     /* Copy fmt -> fmt2, converting ~ escape codes to % escape codes.
     ** Set zone=1 if fmt has a ~Z.
@@ -322,7 +368,7 @@ scheme_value format_date(const char *fmt, int sec, int min, int hour,
 	    char c = *++p;
 	    if( ! c ) {
 		Free(fmt2);
-		return SCHTRUE;	/* % has to be followed by something. */
+		return S48_FALSE;	/* % has to be followed by something. */
 		}
 	    else if( c == '~' ) {
 		*q++ = '~';
@@ -362,12 +408,12 @@ scheme_value format_date(const char *fmt, int sec, int min, int hour,
     *q++ = 'x'; *q = '\0'; /* Append the guard "x" suffix and nul-terminate. */
 
     /* Fix up the time-zone if it is being used and the user passed one in. */
-    if( zone && STRINGP(tz) ) {
-	oldenv = make_newenv(tz, newenv);
+    if( zone && S48_STRING_P(sch_tz) ) {
+	oldenv = make_newenv(sch_tz, newenv);
 	if( !oldenv ) {
 	    int err = errno;
 	    Free(fmt);
-	    return ENTER_FIXNUM(err);
+	    s48_raise_os_error_1(errno, sch_fmt);
 	    }
 	}
 
@@ -383,17 +429,17 @@ scheme_value format_date(const char *fmt, int sec, int min, int hour,
 	}
     target[result_len-1] = '\0'; /* Flush the trailing "x". */
 #endif
-    *ans = target;
+    sch_ans = s48_enter_string(target);
     Free(fmt2);
     if( oldenv ) revert_env(oldenv);
-    return SCHFALSE;
+    return sch_ans;
 
 lose:
     /* We lost trying to allocate space for the strftime() target buffer. */
     {int err = errno;
      if( oldenv ) revert_env(oldenv); /* Clean up */
      Free(fmt2);
-     return ENTER_FIXNUM(err);
+     s48_raise_os_error_1(err, sch_fmt);
      }
 }
 
@@ -422,3 +468,12 @@ char *tzname_loser(struct tm *dp)
 ** gettimeofday() returns -1/errno
 ** localtime() & gmtime() don't error.
 */
+
+void s48_init_time(void)
+{
+    S48_EXPORT_FUNCTION(time_plus_ticks);
+    S48_EXPORT_FUNCTION(scheme_time);
+    S48_EXPORT_FUNCTION(date2time);
+    S48_EXPORT_FUNCTION(time2date);
+    S48_EXPORT_FUNCTION(format_date);
+}
